//---------------------------------------------------------------------
// SPACE TAPPER: game.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game Engine
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int		step;
static int      pause_tiles_start;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

static void		game_init_videodata(void)
{
    const ts_level *level = game.state.level;
    
	// Setup video mode 0, disable all display
	REG_DISPCNT = REG_DISPCNT_MODE_0 | REG_DISPCNT_OBJ_MAP_1D;
				   		   
	// Copy Background Palette
	if (level->special == GAME_SPECIAL_ENTERTAINERS_1 || level->special == GAME_SPECIAL_ENTERTAINERS_2)
	{
		palette_copy((u16 *)entertainers_Palette, MEM_PAL_BG + 3, 16);
		palette_copy(level->layout->layout_palette, MEM_PAL_BG + 3 + 16, 256 - 3 - 16);
	}
	else
	{
		palette_copy(level->layout->layout_palette, MEM_PAL_BG + 3, 256 - 3);
	}

	// Copy Background Tiles
	vram_bg_start(MEM_VRAM_BG_TILEx(64 * 2));
	vram_bg_copy(level->layout->layout_tiles, level->layout->layout_tiles_size);
	vram_bg_copy_block_256(level->layout->layout_bars_start,  &game.block_bar_start,  4, 3);
	vram_bg_copy_block_256(level->layout->layout_bars_repeat, &game.block_bar_repeat, 4, 3);
	//if (game.state.level->special == GAME_SPECIAL_ENTERTAINERS_2)
	//	vram_bg_copy_block_256((u16 *)l3_layout_platform_Bitmap, &game.block_platform, 10, 10);

    // Additionnal part
    if (level->level_idx == 6 || level->level_idx == 7)
        vram_bg_copy_block_256((u16 *)l5_layout_part_Bitmap, &game.block_additionnal.layout5_part, 5, 3);

	// Setup background 0 (Layout)
	// -- Priority 3, 256 Colors, 32x32 tiles, Screen base block @ 0x6000E000, Character base block 0
	background_init(&game.bg0, 0, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
				  			 	  REG_BGxCNT_PRIORITY_Set(3) | REG_BGxCNT_SCREEN_BASE_Set(27));

	// Setup background 1 (Bars)
	// -- Priority x, 256 Colors, 32x32 tiles, Screen base block @ 0x6000E800, Character base block 0
	// Note: Priority is dynamically switched between 1 & 2 in-game.
	background_init(&game.bg1, 1, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(2) | REG_BGxCNT_SCREEN_BASE_Set(28));
	
	// Setup background 2 (Entertainers & Misc)
	// -- Priority 2, 256 Colors, 32x32 tiles, Screen base block @ 0x6000F000, Character base block 0
	background_init(&game.bg2, 2, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(2) | REG_BGxCNT_SCREEN_BASE_Set(29));

	// Setup background 3 (Interface)
	// -- Priority 0, 256 Colors, 32x32 tiles, Screen base block @ 0x6000F800, Character base block 0
	background_init(&game.bg3, 3, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(0) | REG_BGxCNT_SCREEN_BASE_Set(31));

    if (level->level_idx == 6 || level->level_idx == 7)
    {
        game.bg2.cnt = (game.bg2.cnt & ~REG_BGxCNT_PRIORITY_Mask) | REG_BGxCNT_PRIORITY_Set(0);
        background_sync_cnt(&game.bg2);
    }
	
	// Allocate & Copy Sprites Tiles
	vram_obj_start(MEM_VRAM_OBJ);
	player_init_videodata();
	beers_init_videodata();
    money_init_videodata();
    tappers_init_videodata();
	customers_init_videodata();
	
	// Setup Pause video data (Interface)
	palette_copy_16(pause_Palette, MEM_PALx_OBJ(PAUSE_PALETTE_IDX));
    pause_tiles_start = vram_obj_copy(pause_Bitmap, 1 * 5 * TILE_SIZE_16);

	// Setup Lives video data(Interface)
	lives_init_videodata();
	
	// Allocate & Copy Tiles for Level Special Feature(s)
	switch (game.state.level->special)
	{
		case	GAME_SPECIAL_ENTERTAINERS_1:
		case	GAME_SPECIAL_ENTERTAINERS_2:
				entertainers_init_videodata();
				break;
		case	GAME_SPECIAL_TUBEALIEN:
				tubealien_init_videodata();
				break;
	}

    // Setup Alpha Settings for semi-transparent Sprites (Beers)
	REG_BLDCNT =  REG_BLDCNT_TARGET2_BG0 | REG_BLDCNT_TARGET2_BG1 | REG_BLDCNT_TARGET2_BG2 | REG_BLDCNT_EFFECT_ALPHA;
	REG_BLDALPHA = REG_BLDALPHA_EVA_Set (13) | REG_BLDALPHA_EVB_Set (3);
}

static void		game_init(void)
{ 
	// Setup Game Variables
    game.score_displayed = game.state.score;
    game.paused = NO;
	game.update_flags = GAME_UPDATE_ALL;
	game.keys = 0x0000;

    // Clear Sprites
    sprites_clear();

	// Clear Background Maps to their original state
	if (game.state.level->layout_map_x_flipped)
		background_map_copy (&game.bg0, game.state.level->layout->layout_map, BG_MAP_FLIP_H);
	else		
		background_map_copy (&game.bg0, game.state.level->layout->layout_map, 0);
	background_map_clear(&game.bg1, 0x0000);
	background_map_clear(&game.bg2, 0x0000);
	background_map_clear(&game.bg3, 0x0000);

	// Setup Background Scrolling
	game.bg0.scroll_y = 0;
	game.bg1.scroll_y = 4;
	game.bg2.scroll_y = 4;
	game.bg3.scroll_x = game.bg3.scroll_y = -4;
	background_sync_scroll(&game.bg0);
	background_sync_scroll(&game.bg1);
	background_sync_scroll(&game.bg2);
	background_sync_scroll(&game.bg3);

	// Initialize Player
	player_init();

	// Clear Entities
	entities_clear();
	bars_entities_clear();

	// Setup Tappers Sprites
	tappers_init();
	
	// Setup Lives Sprite & Value
	lives_init();
	
	// Setup Level Special Feature
	switch (game.state.level->special)
	{
		case	GAME_SPECIAL_ENTERTAINERS_1:
		case	GAME_SPECIAL_ENTERTAINERS_2:
				entertainers_init();
				break;
		case	GAME_SPECIAL_TUBEALIEN:
				tubealien_init();
				break;
	}

    if (game.state.level->level_idx == 6 || game.state.level->level_idx == 7)
    {
    	// Blit additionnal parts
		background_block_draw(&game.bg2, &game.block_additionnal.layout5_part, 0, 8, BG_MAP_FLIP_NONE);
		background_block_draw(&game.bg2, &game.block_additionnal.layout5_part, -2, 14, BG_MAP_FLIP_NONE);
		background_block_draw(&game.bg2, &game.block_additionnal.layout5_part, 30, 11, BG_MAP_FLIP_H);
    }

	// Draw Bars in Background 1
	bars_draw();
	
	// Draw Score in Background 3
	font_bg_printf(&game.bg3, game.font_tiles_start, 0, 0, "SCORE\n%06d", game.state.score);
}

void			game_restart(void)
{
	// Restart level sequence and initialize various level related data
	level_start(game.state.level->level_idx);
	game_init ();
}

//---------------------------------------------------------------------

// game.state must be set prior to calling this section
void			game_main_start(va_list args)
{
	int			i;

	// Initialize Game & Level Specific Video Data
	game_init_videodata();

	// (Re)Start Level
	game_restart();

	// Setup & Enable interrupts
	intrs_one_enable(INTR_VCOUNT);
	intrs_one_handler_set(INTR_VCOUNT, intrs_vcount_handler);
	intrs_vcount_handler_setup_callback(bars_vcount_handler);
	intrs_vcount_handler_setup_clear();
	for (i = 0; i < BAR_MAX; i++)
		intrs_vcount_handler_setup_addline(FINT(bars[i].yf, 4) + 5, (void *)(i + 1));
	intrs_vcount_handler_setup_addline(200, (void *)0);
	intrs_vcount_handler_enable();
	intrs_cpu_enable();
	
	// Execute step 0 (window fade-in) immediately, to avoid a one-frame flickering
	step = 0;
	game_main_update();
}

// FIXME: should not be done that way! the start/restart things are a mess...
void            game_main_restart(void)
{
	step = 4;
}

void			game_main_next_level(void)
{
	step = 6;
}

void			game_main_update(void)
{
	switch (step)
	{
		case 0: // Level Introduction Init
		{
			step++;
			//break;
		}
		case 1: // Level Introduction
		{
			step++;
			break;
		}
		case 2: // Window Open
		{
			// Start Window Opening Effect
			section_call(SECTION_EFFECTS, EFFECT_P0_WINDOW_IN, 60, RES_WIDTH / 2, RES_HEIGHT / 2, RES_WIDTH / 2, RES_HEIGHT / 2);
			// Enable Display
			REG_DISPCNT |= REG_DISPCNT_BG0_ENABLE |		// Enable Background 0 display 
					   	   REG_DISPCNT_BG1_ENABLE |		// Enable Background 1 display
					   	   REG_DISPCNT_BG2_ENABLE |		// Enable Background 2 display
					   	   REG_DISPCNT_BG3_ENABLE |		// Enable Background 3 display
				   		   REG_DISPCNT_OBJ_ENABLE;		// Enable Sprites display
			step++;
			break;
		}
		case 3: // GAME Running
		{
		    // Check for pause
	        if ((game.keys & REG_KEYINPUT_KEY_START) && !(game.keys_old & REG_KEYINPUT_KEY_START))
	        {
	            if (game.paused)
	            {
	                game.paused = NO;
	            }
	            else
	            {
	                game.paused = YES;
	            }
	        }
            if (game.paused)
                break;
		    
			// Update Score
			if (game.score_displayed != game.state.score)
			{
		    	// Increment displayed score by 9, so we'll show all different digits
		        if (game.score_displayed < game.state.score)
					game.score_displayed += 9;
				if (game.score_displayed > game.state.score)
					game.score_displayed = game.state.score;
				font_bg_printf(&game.bg3, game.font_tiles_start, 0, 1, "%06d", game.score_displayed);
			}
		            	
			// Update Game
			tappers_update();
			if (game.update_flags & GAME_UPDATE_ENTITIES)
		    	bars_entities_update();
			if (game.update_flags & GAME_UPDATE_PLAYER)
				player_update();
			if (game.update_flags & GAME_UPDATE_SEQUENCE)
				level_sequencer_update();
				
			// Check for end of level
			if (sequencer.completed)
				if (entities_counter[ENTITY_TYPE_CUSTOMER] == 0)
					if (player.state->state != PLAYER_STATE_VICTORY)
						player_victory();
				
			// Update Level Special feature
			switch (game.state.level->special)
			{
				case	GAME_SPECIAL_ENTERTAINERS_1:
				case	GAME_SPECIAL_ENTERTAINERS_2:
						entertainers_update();
						break;
				case	GAME_SPECIAL_TUBEALIEN:
						tubealien_update();
						break;
			}
			break;
		}
		case 4: // Windows close
		{
			effects_setup_closure(NO, NULL); // Disable Effects closure (since we want another effect to follow this one)
			section_call(SECTION_EFFECTS, EFFECT_P0_WINDOW_OUT, 60, RES_WIDTH / 2, RES_HEIGHT / 2, RES_WIDTH / 2, RES_HEIGHT / 2);
			step++;
			break;
		}
		case 5: // Restart
		{
			effects_setup_closure(YES, NULL); // Re-enable Effects closure (the next effect on step 0 will close properly)
			if (game.state.lives == 0)
			{
				section_jump(SECTION_INTERFACE_GAME_OVER);
				return;
			}
			else
			{
				game_restart();
				step = 0;
			}
			break;
		}
		case 6: // Windows close
		{
			effects_setup_closure(NO, NULL); // Disable Effects closure (since we want another effect to follow this one)
			section_call(SECTION_EFFECTS, EFFECT_P0_WINDOW_OUT, 60, RES_WIDTH / 2, RES_HEIGHT / 2, RES_WIDTH / 2, RES_HEIGHT / 2);
			step++;
			break;
		}
		case 7: // Next level
		{
			effects_setup_closure(YES, NULL); // Re-enable Effects closure (the next effect on step 0 will close properly)
			level_start(game.state.level->level_idx + 1);
			section_jump(SECTION_INTERFACE_LEVEL_END);
			return;
		}
	}
}

void			game_main_stop(void)
{
	REG_DISPCNT &= ~(REG_DISPCNT_BG0_ENABLE |	// Disable Background 0 display 
			   	     REG_DISPCNT_BG1_ENABLE |	// Disable Background 1 display
			   	     REG_DISPCNT_BG2_ENABLE |	// Disable Background 2 display
			   	     REG_DISPCNT_BG3_ENABLE |	// Disable Background 3 display
		   		     REG_DISPCNT_OBJ_ENABLE);	// Disable Sprites display
}

//---------------------------------------------------------------------
