//---------------------------------------------------------------------
// SPACE TAPPER: game_bars.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Bars
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			bars_draw(void)
{
	int			i;

	// Draw all bars on BG1
	for (i = 0; i < BAR_MAX; i++)
	{
		t_bar *bar = &bars[i];
		s16 yt  =  (FINT(bar->yf,  4) / 8) + 1; 
		// FIXME: some Y scrolling mess for background 1 was done when attempting to add the entertainers
		// The basic scrolling went from -4 to 4, and as a result bars needed to be moved by one tile as well
		
		if (bar->direction == DIRECTION_LEFT)
		{
			s16 x1t = ((FINT(bar->xf1, 4) - 5) / 8); // 5 is the offset in *layout_bars_start.bmp
			s16 x2t =  (FINT(bar->xf2, 4) / 8);
			background_block_draw(&game.bg1, &game.block_bar_start, x1t, yt, BG_MAP_FLIP_NONE);
			x1t += 4;
			while (x1t <= x2t)
			{
				background_block_draw(&game.bg1, &game.block_bar_repeat, x1t, yt, BG_MAP_FLIP_NONE);
				x1t += 4;
			}
		}
		else // bars->direction == DIRECTION_RIGHT
		{
			s16 x1t = ((FINT(bar->xf1, 4) + 5) / 8); // 5 is the offset in *layout_bars_start.bmp
			s16 x2t =  (FINT(bar->xf2, 4) / 8);
			background_block_draw(&game.bg1, &game.block_bar_start, x1t, yt, BG_MAP_FLIP_H);
			x1t -= 4;
			while (x1t >= x2t)
			{
				background_block_draw(&game.bg1, &game.block_bar_repeat, x1t, yt, BG_MAP_FLIP_H);
				x1t -= 4;
			}
		}
	}
}

void			bars_entities_clear(void)
{
	int			i;
	
	// Clear bars entities list
	for (i = 0; i < BAR_MAX; i++)
		bars_entities[i] = NULL;
}

void            bars_entities_active_set(int type, int active)
{
    int         i;
    t_entity *  entity;
    
    for (i = 0; i < BAR_MAX; i++)
    {
        entity = bars_entities[i];
        while (entity != NULL)
        {
            if (type == ENTITY_TYPE_ALL || type == entity->type)
                entity->active = active;
            entity = entity->next;
        }
    }
}

void			bars_entities_update(void)
{
	int			i;
	t_entity *	entity;
	t_entity *	next;
	
	for (i = 0; i < BAR_MAX; i++)
	{
		entity = bars_entities[i];
		// dprintf("bars_update . %d\n", i);
		while (entity != NULL)
		{
			next = entity->next;
			if (entity->active)
			{
		    	// dprintf(" - entity: type %d\n", entity->type);
		    	// FIXME: use function pointers ? that's where I'd like to use C++...
		    	switch (entity->type)
		    	{
		    	    case ENTITY_TYPE_CUSTOMER:  customer_update (entity);   break;
		    	    case ENTITY_TYPE_BEER:      beer_update     (entity);   break;
		    	    case ENTITY_TYPE_MONEY:     money_update    (entity);   break;
		    	}
		    }
			entity = next;
		}
	}
}

void			bars_entities_update_priority(void)
{
	int			i;
	int         priority;
	t_entity *	entity;
	
	priority = 2;
	for (i = 0; i < BAR_MAX; i++)
	{
		if (i == player.state->bar_idx + 1)
			priority = 1;
		entity = bars_entities[i];
		while (entity != NULL)
		{
		    if (entity->type == ENTITY_TYPE_BEER && entity->state == BEER_STATE_HEAVY_DRINKER_THROWN)
		    {
		        // Do not touch to priority in this case
		        // FIXME: Hardcoded
		        // We may want to add a specific flag for this purpose
		    }
		    else
		    {
			    sprites[entity->sprite_idx].oam.priority = priority;
			}
			entity = entity->next;
		}
	}
}

//---------------------------------------------------------------------

int				bar_entities_count(int bar_idx, int type)
{
	int			count;
	t_entity *	entity;
	
	count = 0;
	entity = bars_entities[bar_idx];
	while (entity != NULL)
	{
		if (type == ENTITY_TYPE_ALL || type == entity->type)
			count++;
		entity = entity->next;
	}
	return (count);
}

//---------------------------------------------------------------------

// See game.c where the handler hooks are installed (one per bar)

void			bars_vcount_handler(void *userdata)
{
	// Display Start (& Entertainers/Alien)
	if (userdata == NULL)
	{
		background_set_priority(&game.bg1, 2);
		REG_BG1HOFS = 0;
		return;
	}

	// Bars Priority
	int bar_idx = (int)userdata - 1;
	REG_BG1HOFS = bars[bar_idx].x_hoffset; // -((bars[bar_idx].x1 - 5) & 7);
	if (bar_idx <= player.state->bar_idx)
		background_set_priority(&game.bg1, 2);
	else
		background_set_priority(&game.bg1, 1);
  }

//---------------------------------------------------------------------
