//---------------------------------------------------------------------
// SPACE TAPPER: game_entity.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Entities
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static t_entity	entities[ENTITIES_MAX];
u8				entities_counter[ENTITY_TYPE_MAX];

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			entities_clear(void)
{
	int			i;
	
	for (i = 0; i < ENTITIES_MAX; i++)
		entities[i].type = ENTITY_TYPE_UNUSED;
	for (i = 0; i < ENTITY_TYPE_MAX; i++)
		entities_counter[i] = 0;
}

#ifdef DEBUG
void			entities_print(void)
{
	int			i, j;
	t_entity *	entity;
	t_entity *	next;

	dprintf("player.beer = %p, player.beer_catched = %p\n", player.beer, player.beer_catched);
	dprintf("player.state->state = %d, player.state->sub_state = %d\n", player.state->state, player.state->sub_state);
	dprintf("player.state->xf = %d.%d, player.state->yf = %d.%d\n", FINT(player.state->xf,4), FDEC(player.state->xf,4), FINT(player.state->yf,4), FDEC(player.state->yf,4));
	for (i = 0; i < BAR_MAX; i++)
	{
		entity = bars_entities[i];
		dprintf("BAR %d\n", i);
		j = 0;
		while (entity != NULL)
		{
			next = entity->next;
			if (entity->type != ENTITY_TYPE_CUSTOMER)
				dprintf("- entity %d @ %p, type %d, active %d, state = %d, sub_state = %d, x = %d.%d, y = %d.%d\n", j, entity, entity->type, entity->active, entity->state, entity->sub_state, FINT(entity->xf, 4), FDEC(entity->xf, 4), FINT(entity->yf, 4), FDEC(entity->yf, 4));
			entity = next;
			j++;
		}
	}
}
#endif /* DEBUG */

//---------------------------------------------------------------------

t_entity *      entity_new(int type, int bar_idx)
{
	int		i;
	
	// Find an unused entity
	for (i = 0; i < ENTITIES_MAX; i++)
		if (entities[i].type == ENTITY_TYPE_UNUSED)
			break;
	if (i == ENTITIES_MAX)
		return (NULL);
		
	// Update counters
	entities_counter[type]++;
	entities_counter[ENTITY_TYPE_UNUSED]--;

    // Add entity to bar list
	t_entity *e = &entities[i];
	e->next = bars_entities[bar_idx];
	bars_entities[bar_idx] = e;
	
	// Setup basic variables
	e->type = type;
	e->bar_idx = bar_idx;
	
	return (e);
}

void			entity_delete(t_entity *entity)
{
	t_entity *	prev = NULL;
	t_entity *	list = bars_entities[entity->bar_idx];

	// dprintf("entity_delete()...\n");
	
	// Update counters
	entities_counter[entity->type]--;
	entities_counter[ENTITY_TYPE_UNUSED]++;

	while (list)
	{
		if (list == entity)
			break;
		prev = list;
		list = list->next;
	}
	if (prev)
		prev->next = entity->next;
	else
		bars_entities[entity->bar_idx] = entity->next;

	entity->type = ENTITY_TYPE_UNUSED;
	// free(entity);
}

//---------------------------------------------------------------------
