//---------------------------------------------------------------------
// SPACE TAPPER: game_entity.h
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Entities
//---------------------------------------------------------------------

//---------------------------------------------------------------------

#define ENTITIES_MAX			    	(64)

#define ENTITY_TYPE_UNUSED		    	(0)
#define ENTITY_TYPE_ALL                 (0) // for bars_entities_active_set()
#define ENTITY_TYPE_CUSTOMER	    	(1)
#define ENTITY_TYPE_BEER		    	(2)
#define ENTITY_TYPE_MONEY			    (3)
#define ENTITY_TYPE_MAX					(4)

//---------------------------------------------------------------------

// Sprites & Background Priority

// Priorities in the game are defined as follow
 // BAR 0
   // Customer
   // Customer + Beer (when drinking)
   // Beer
   // Money
   // Player
 // BAR 1
   // ...
 // BAR 2
   // ...
 // BAR 3
   // ...

#define ENTITY_SPR_PRIORITY(BAR_IDX)                    (((BAR_IDX) > player.state->bar_idx + 1) ? 1 : 2)

#define ENTITY_SUB_PRIORITY_CUSTOMER_WATCHING(BAR_IDX)  (12 + ((BAR_IDX) * 8))
#define ENTITY_SUB_PRIORITY_CUSTOMER(BAR_IDX)           (13 + ((BAR_IDX) * 8))
#define ENTITY_SUB_PRIORITY_CUSTOMER_DRINKING(BAR_IDX)  (14 + ((BAR_IDX) * 8))
#define ENTITY_SUB_PRIORITY_BEER(BAR_IDX)               (15 + ((BAR_IDX) * 8))
#define ENTITY_SUB_PRIORITY_MONEY(BAR_IDX)              (16 + ((BAR_IDX) * 8))
#define ENTITY_SUB_PRIORITY_TAPPER(BAR_IDX)             (17 + ((BAR_IDX) * 8))
#define ENTITY_SUB_PRIORITY_PLAYER(BAR_IDX)             (18 + ((BAR_IDX) * 8))

//---------------------------------------------------------------------

typedef struct		s_entity
{
	u8				type;				/* Item type								*/
	u8				sub_type;			/* Sub-type, item dependant					*/
	u8              active;             /* Active (is updated if set)               */
	u8				state;				/* Entity state								*/
	u8              sub_state;          /* Sub-state, item dependant                */
	u8              direction;          /* Entity Direction                         */
	u8				sprite_idx;			/* Sprite index								*/
	u8				bar_idx;			/* Bar index								*/
	fs16			xf;					/* X Position (in bar->pos_x interval)		*/
	fs16			yf;					/* Y Position								*/
	t_animator		anim;				/* Animator for the sprite					*/
	union
	{
		struct
		{
			u8		thirstyness;		/* Customer thirstyness						*/
			u8      amount_drunk;       /* Number of beers drinken                  */
			u8		move_step_delay;	/* Time before next move step occurs		*/
			u8		move_step_left;		/* Number of move left to do for step		*/
			struct s_entity *beer;		/* Beer currently drinking					*/
		}	customer;
		struct
		{
			u8		filled;				/* Beer Filled (0: no beer, >0: filled) 	*/
			u8		falling_step;       /* Falling step                             */
			u8      rotation_idx;       /* Rotation index used when falling         */
			fs16	return_speed;		/* Beer speed when being returned			*/
		}	beer;
		struct
		{
			u16		amount;             /* Money value                              */
			u16     life_left;          /* Amount of frame left until it disappear  */
		}	money;
	};
	void *			next;				/* Pointer to next item in list 			*/
}				    t_entity;

//---------------------------------------------------------------------

extern u8	entities_counter[ENTITY_TYPE_MAX];

//---------------------------------------------------------------------

void		entities_clear(void);
#ifdef DEBUG
void		entities_print(void);
#endif

t_entity *	entity_new(int type, int bar_idx);
void		entity_delete(t_entity *entity);

#define		ENTITY_OAM_SYNC_X(ENT)		sprites[ENT->sprite_idx].oam.x = FINT(ENT->xf, 4)
#define		ENTITY_OAM_SYNC_Y(ENT)		sprites[ENT->sprite_idx].oam.y = FINT(ENT->yf, 4)
#define		ENTITY_OAM_SYNC(ENT)		MACRO_START t_sprite_oam *SO = &sprites[ENT->sprite_idx].oam; SO->x = FINT(ENT->xf, 4); SO->y = FINT(ENT->yf, 4); MACRO_END

//---------------------------------------------------------------------
