//---------------------------------------------------------------------
// SPACE TAPPER: game_entity_beers.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game	- Entity - Beers handling
//---------------------------------------------------------------------
// Note: use 8 sprites rotation slots (0-7)
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int		beers_rotation_idx = 0;	// 0-7
static int		beers_tiles_start;

// Relative offsets per frame when beer is thrown by the heavy drinker
static fs16     beers_heavy_drinker_thrown_offset_table[16*2] = 
{
    FFIX4(-2.0), FFIX4(-3.0),
    FFIX4(-2.0), FFIX4(-2.0),
    FFIX4(-1.9), FFIX4(-1.6),
    FFIX4(-1.9), FFIX4(-1.2),
    FFIX4(-1.8), FFIX4(-1.0),
    FFIX4(-1.8), FFIX4( 0.8),
    FFIX4(-1.7), FFIX4( 0.5),
    FFIX4(-1.7), FFIX4( 0.0), // Max point
    FFIX4(-1.6), FFIX4( 0.5),
    FFIX4(-1.6), FFIX4( 0.8),
    FFIX4(-1.5), FFIX4( 1.0),
    FFIX4(-1.4), FFIX4( 1.2),
    FFIX4(-1.3), FFIX4( 1.6),
    FFIX4(-1.2), FFIX4( 2.0),
    FFIX4(-1.1), FFIX4( 3.0),
    FFIX4(-1.0), FFIX4( 3.0)
};

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			beers_init_videodata(void)
{
	palette_copy_16((u16 *)beer_Palette, MEM_PALx_OBJ(BEER_PALETTE_IDX));
	beers_tiles_start =	vram_obj_copy((u16 *)beer_Bitmap, sizeof (beer_Bitmap));
}

//---------------------------------------------------------------------

t_entity *		beer_new(int bar_idx, int x, int y)
{
	// Create entity and add to	bar	list
	t_entity *beer = entity_new(ENTITY_TYPE_BEER, bar_idx);

	// Setup Entity
	beer->active = YES;
	beer->sub_type = 0;
	beer->xf = IFIX4(x);
	beer->yf = IFIX4(y);
	beer->direction = DIRECTION_OPPOSITE(bars[bar_idx].direction);
	beer->state	= BEER_STATE_FILLING;
	beer->sub_state	= 0;
	beer->beer.filled =	0;

	// Setup Sprite
	beer->sprite_idx = sprite_new(ENTITY_SUB_PRIORITY_BEER(bar_idx));
	sprites[beer->sprite_idx].oam.attr0	= SPR_A0_COLOR_16 |	SPR_A0_SHAPE_TALL | SPR_A0_MODE_TRANS; // | SPR_A0_ROTATE | SPR_A0_DOUBLE;
	sprites[beer->sprite_idx].oam.attr1	= SPR_A1_SIZE_8 | ((beer->direction == DIRECTION_LEFT) ? SPR_A1_FLIP_H : SPR_A1_FLIP_NONE);
	sprites[beer->sprite_idx].oam.attr2	= SPR_A2_PRIORITY_Set(ENTITY_SPR_PRIORITY(bar_idx))	| SPR_A2_PALETTE_Set(BEER_PALETTE_IDX) | SPR_A2_CHAR_Set(beers_tiles_start);
	ENTITY_OAM_SYNC(beer);
	// sprite_oam_rotate(0,	-10, 0x100,	0x100);

	// Setup Animation
	animator_init(&beer->anim, (ts_anims_set *)data_beer_anim_set, &sprites[beer->sprite_idx],
					ANIMATOR_MODE_OAM_CHANGE_IDX, (void	*)beers_tiles_start, NULL);
	animator_start(&beer->anim,	BEER_ANIM_EMPTY, 0);

	return (beer);
}

void			beer_delete(t_entity *beer)
{
	sprite_delete(beer->sprite_idx);
	entity_delete(beer);
}

void			beer_update(t_entity *beer)
{
	int			dir_left = (beer->direction == DIRECTION_LEFT);
	
	switch (beer->state)
	{
		case	BEER_STATE_FILLING:
		{
				break;
		}
		case	BEER_STATE_THROWN:
		{
				// Move beer
				beer->xf += (dir_left ? -BEER_SPEED_THROWN : BEER_SPEED_THROWN);

				// Get beer bounding box data (Currently hardcoded to 0->7)
				t_boundingbox bb_beer;
				bb_beer.xf1 = beer->xf + IFIX4(0);
				bb_beer.xf2 = beer->xf + IFIX4(7);

				// Check reaching of end of screen
				int d = boundingbox_cmpx(&bb_beer, bars[beer->bar_idx].xf2 + (dir_left ? -BEER_LENGTH_BARX2_TO_CRASH_RIGHT : BEER_LENGTH_BARX2_TO_CRASH_RIGHT));
				if ((dir_left && d <= 0) || (!dir_left && d >= 0))
				{
					beer->state	= BEER_STATE_FALLING_RIGHT;
					beer->sub_state	= BEER_SUB_STATE_FALLING_RIGHT_INIT;
					beer->beer.falling_step	= 0;
					ENTITY_OAM_SYNC_X(beer);
					return;
				}

				// Check for collision with a customer
				t_entity *entity;
				for	(entity	= bars_entities[beer->bar_idx];	entity != NULL;	entity = entity->next)
				{
					if (entity->type !=	ENTITY_TYPE_CUSTOMER ||	entity->state != CUSTOMER_STATE_NORMAL)
						continue;
					t_boundingbox *bb_customer = animator_get_current_boundingbox(&entity->anim) + DIRECTION_TO_OFFSET(entity->direction);
					if (boundingboxes_overlap_center_o(&bb_beer, bb_customer, entity->xf, dir_left ? IFIX4(-4) : IFIX4(4)))
					{
						// Set beer drinking state
						beer->state	= BEER_STATE_DRINKING;
						// Disable beer alpha blending when positionned on the customer (or it'll look strange)
						sprites[beer->sprite_idx].oam.mode = SPR_MODE_NORMAL;
						// Round entity position
						// (This is needed for the computation below, to avoid differences between customers & beers
						//  Ex: entity->xf = 10.2, bb_customer average = 100.7 ~ 100 -> 110.9 ~ 110 (OK)
						//      entity->xf = 10.5, bb_customer average = 100.7 ~ 100 -> 111.2 ~ 111 (Not OK)
						entity->xf = FTRUNC4(entity->xf);
						// Adjust beer to customer position
						beer->xf = entity->xf + ((bb_customer->xf1 + bb_customer->xf2) >> 1) 
						         + (dir_left ? customers_def[entity->sub_type].beer_shift_l : customers_def[entity->sub_type].beer_shift_r);
						customer_drink(entity, beer);
						break;
					}
				}
				ENTITY_OAM_SYNC_X(beer);
				break;
		}
		case	BEER_STATE_DRINKING:
		{
				// Nothing
				animator_update(&beer->anim);
				break;
		}
		case	BEER_STATE_RETURNING:
		{
				// Move beer
				beer->xf += beer->beer.return_speed;
				ENTITY_OAM_SYNC_X(beer);

				// Test if player has taken it
				if (beer_collide_with_player(beer))
				{
					beer_player_catch(beer);
					break;
				}
				
				// Get beer bounding box data (Currently hardcoded to 0->7)
				t_boundingbox bb_beer;
				bb_beer.xf1 = beer->xf + IFIX4(0); // IFIX4(bb_beer->x1);
				bb_beer.xf2 = beer->xf + IFIX4(7); // IFIX4(bb_beer->x2);
				
				// Check reaching of end of bar and falling
				int d = boundingbox_cmpx(&bb_beer, bars[beer->bar_idx].xf1 + (dir_left ? BEER_LENGTH_BARX1_TO_CRASH_LEFT : -BEER_LENGTH_BARX1_TO_CRASH_LEFT));
				if ((dir_left && d <= 0) || (!dir_left && d >= 0))
				{
					beer->state	= BEER_STATE_FALLING_LEFT;
					beer->sub_state	= BEER_SUB_STATE_FALLING_LEFT_INIT;
				}
				break;
		}
		case	BEER_STATE_FALLING_LEFT:
		{
				switch (beer->sub_state)
				{
					case BEER_SUB_STATE_FALLING_LEFT_INIT: // Initialize
					{
						// Pick	up next	beer available rotation	index (8 allocated for beers)
						// FIXME: have a global rotation slot allocater
						beers_rotation_idx = (beers_rotation_idx + 1) &	7;
						beer->beer.rotation_idx	= beers_rotation_idx;
						// Setup rotation
						sprites[beer->sprite_idx].oam.rotate = YES;
						sprites[beer->sprite_idx].oam.doublesize = YES;
						sprites[beer->sprite_idx].oam.hflip = 0;
						sprites[beer->sprite_idx].oam.rotindex = beer->beer.rotation_idx;
						sprite_oam_rotate(beer->beer.rotation_idx, 0);
						// Offset beer due to rotation
						beer->xf += IFIX4(-4); // beer size x / 2
						beer->yf += IFIX4(-8); // beer size y / 2
						ENTITY_OAM_SYNC(beer);
						// Switch to next sub_state
						beer->sub_state++;
						beer->beer.falling_step	= 0;
						break;
					}
					case BEER_SUB_STATE_FALLING_LEFT_FALL: // Fall & Rotate
					case BEER_SUB_STATE_FALLING_LEFT_CATCHED:
					{
						// Steps 0 -> 16
						int	step = ++beer->beer.falling_step;
						if (step <= 16)
						{
							// Move beer (Fall)
							beer->xf += (dir_left ? -BEER_SPEED_CRASH_LEFT_X : BEER_SPEED_CRASH_LEFT_X);
							beer->yf += BEER_SPEED_CRASH_LEFT_Y;
							ENTITY_OAM_SYNC(beer);
							
							// Rotate
							int rotation_angle = (dir_left ? -(step << 2) : (step << 2)) & 0xFF;
							sprite_oam_rotate(beer->beer.rotation_idx, rotation_angle);
							
							// We have 16 steps of rotation, player beer catching is done on step 12.
							if (beer->sub_state != BEER_SUB_STATE_FALLING_LEFT_CATCHED)
							{
								if (beer->bar_idx == player.state->bar_idx && !(player.state->state & (PLAYER_STATE_SWITCHING_DOWN|PLAYER_STATE_SWITCHING_UP)))
								{
									if (step < 11)
									{
										// Test if player has taken it
										if (player.state->xf != bars[beer->bar_idx].xf_player)
											if (beer_collide_with_player(beer))
											{
												beer_player_catch(beer);
												break;
											}
									}
									if (step >= 11)
									{
										// If there is already one beer being catched, count it
										if (player.beer_catched != NULL)
											player_beer_catched_catch();
										// Now see if the player is currently before the point the beer started to fall
										int colliding;
										if (bars[beer->bar_idx].direction == DIRECTION_LEFT)
											colliding = (player.state->xf < bars[beer->bar_idx].xf1 + BEER_LENGTH_BARX1_TO_CRASH_LEFT + 8);
										else
											colliding = (player.state->xf > bars[beer->bar_idx].xf1 - BEER_LENGTH_BARX1_TO_CRASH_LEFT - 8);
										if (colliding)
										{
											player_state_beer_catching(beer);
											beer->sub_state = BEER_SUB_STATE_FALLING_LEFT_CATCHED;
										}
									}
								}
								if (step == 16)
									beer->sub_state = BEER_SUB_STATE_FALLING_LEFT_CRASH_INIT;
							}
						}
						break;
					}
					//case BEER_SUB_STATE_FALLING_LEFT_CATCHED: // Beer Catched Delay
					//{
					//	// Player is in charge of notifying when the beer is catched!
					//	break;
					//}
					case BEER_SUB_STATE_FALLING_LEFT_CRASH_INIT: // Crash Animation Start
					{
						if (game.state.rules & GAME_RULES_LOSE_BEER_FALL_LEFT)
						{
							// Set player in Beer Breaking state
							player_state_beer_breaking(beer->bar_idx);
							// Stop	all	other entities beside current beer
							bars_entities_active_set(ENTITY_TYPE_ALL, NO);
							beer->active = YES;
							game.update_flags &= ~GAME_UPDATE_SEQUENCE;
						}
						animator_start(&beer->anim,	dir_left ? BEER_ANIM_BREAKING_LEFT : BEER_ANIM_BREAKING_RIGHT, 0);
						beer->sub_state++;
						break;
					}
					case BEER_SUB_STATE_FALLING_LEFT_CRASH:	// Crash Animation Play	& Check	End
					{
						animator_update(&beer->anim);
						if (beer->anim.state.loop_count > 0)
						{
							beer->sub_state++;
							beer->beer.falling_step	= 0;
							//game.update_flags	&= ~GAME_UPDATE_ENTITIES;
						}
						break;
					}
					case BEER_SUB_STATE_FALLING_LEFT_CRASH_DELAY: // Delay
					{
						if (++beer->beer.falling_step == 60)
						{
							beer_delete(beer);
							if (game.state.rules & GAME_RULES_LOSE_BEER_FALL_LEFT)
								player_live_lose();
							return;
						}
						break;
					}
				}
				break;
		}
		case	BEER_STATE_FALLING_RIGHT:
		{
				switch (beer->sub_state)
				{
					case BEER_SUB_STATE_FALLING_RIGHT_INIT:
					{
						if (game.state.rules & GAME_RULES_LOSE_BEER_FALL_RIGHT)
						{
							// Stop	all	other entities beside current beer
							bars_entities_active_set(ENTITY_TYPE_ALL, NO);
							beer->active = YES;
							game.update_flags &= ~(GAME_UPDATE_PLAYER |	GAME_UPDATE_SEQUENCE);
						}
						beer->sub_state++;
						break;
					}
					case BEER_SUB_STATE_FALLING_RIGHT_CRASH: // Crashing Effect
					{
						int step = ++beer->beer.falling_step;
						// Slow	down by skipping 3/4 frames
						if (step & 3)
							break;
						step /=	4;
						if (step ==	CRASH_STEP_MAX)
						{
							if (game.state.rules & GAME_RULES_LOSE_BEER_FALL_RIGHT)
							{
								beer->sub_state++;
								beer->beer.falling_step	= 0;
							}
							else
							{
								beer_delete(beer);
							}
						}
						else
						{
							crash_do_step(step);
						}
						break;
					}
					case BEER_SUB_STATE_FALLING_RIGHT_DELAY: // Delay
					{
						if (++beer->beer.falling_step == 60)
						{
							beer_delete(beer);
							player_live_lose();
						}
						break;
					}
				}
				break;
		}
        case    BEER_STATE_HEAVY_DRINKER_THROWN:
        {
		    int	step = ++beer->beer.falling_step;
			if (step < 16)
			{
				// Move beer (Thrown)
				fs16 xd = beers_heavy_drinker_thrown_offset_table[step*2+0];
				fs16 yd = beers_heavy_drinker_thrown_offset_table[step*2+1];
				beer->xf += (dir_left ? xd : -xd);
				beer->yf += yd;
				ENTITY_OAM_SYNC(beer);
							
				// Rotate
				int rotation_angle = (dir_left ? -10-(step << 1) : 10+(step << 1)) & 0xFF;
				sprite_oam_rotate(beer->beer.rotation_idx, rotation_angle);
			}
			else
			{
			    // Self destroy
			    beer_delete(beer);
			    return;
			}

            break;
        }
	}
}

//---------------------------------------------------------------------

int				beer_collide_with_player(t_entity *beer)
{
	// Cannot collide if they are not on the same bar
	if (beer->bar_idx != player.state->bar_idx)
		return (NO);

	// Get beer bounding box data (Currently hardcoded to 0->7)
	t_boundingbox bb_beer;
	bb_beer.xf1 = beer->xf + IFIX4(0); // IFIX4(bb_beer->x1);
	bb_beer.xf2 = beer->xf + IFIX4(7); // IFIX4(bb_beer->x2);

	// Get player bounding box data
	t_boundingbox *bb_player = animator_get_current_boundingbox(&player.anim);
	t_boundingbox bb_player_default;
	if (*(fs32 *)bb_player)
	{
		bb_player += DIRECTION_TO_OFFSET(player.state->direction);
	}
	else
	{
		// Set default player bounding box (11->25) if we have none available for this frame
		// FIXME: this should go in player!
		if (player.state->direction == DIRECTION_LEFT)
		{
			bb_player_default.xf1 = IFIX4(11);
			bb_player_default.xf2 = IFIX4(25);
		}
		else
		{
			bb_player_default.xf1 = IFIX4(6);
			bb_player_default.xf2 = IFIX4(20);
		}
		bb_player = &bb_player_default;
	}
				
	// Now perform the actual collision test
	if (boundingboxes_overlap_o(&bb_beer, bb_player, player.state->xf))
		return (YES);
	return (NO);
}

void			beer_player_throw(t_entity *beer)
{
	// Set THROWN state
	beer->state = BEER_STATE_THROWN;
	// Position beer correctly on the bar
	t_bar *bar = &bars[beer->bar_idx];
	beer->yf = bar->yf + BEER_SPR_OFFSET_BAR_Y;
	beer->xf = bar->xf1 + ((bar->direction == DIRECTION_LEFT) ? BEER_SPR_OFFSET_BAR_X : -BEER_SPR_OFFSET_BAR_X - IFIX4(BEER_SPR_SIZE_X));
	ENTITY_OAM_SYNC(beer);
}

void			beer_customer_return(t_entity *beer, fs16 speed)
{
	// Set RETURNING state
	beer->state = BEER_STATE_RETURNING;
	// Disable beer alpha blending when customer throw the beer away
	sprites[beer->sprite_idx].oam.mode = SPR_MODE_TRANS;
	// Switch beer direction
	beer->direction = DIRECTION_OPPOSITE(beer->direction);
	// sprites[beer->sprite_idx].oam.hflip = 1;
	// Set returning speed
	beer->beer.return_speed = (beer->direction == DIRECTION_LEFT ? -speed : speed);
}

void            beer_customer_heavy_drinker_throw_back(t_entity *beer)
{
    // Set HEAVY_DRINKER_THROWN state
    beer->state = BEER_STATE_HEAVY_DRINKER_THROWN;
	beer->beer.falling_step	= 0;
    
    // Specific priority (behind bars and other entities)
//    sprites[beer->sprite_idx].oam.priority = 3;
	sprite_priority_set(beer->sprite_idx, ENTITY_SUB_PRIORITY_BEER(beer->bar_idx));

	// Pick	up next	beer available rotation	index (8 allocated for beers)
	// FIXME: have a global rotation slot allocater
	beers_rotation_idx = (beers_rotation_idx + 1) &	7;
	beer->beer.rotation_idx	= beers_rotation_idx;
    // Setup rotation fields
	sprites[beer->sprite_idx].oam.rotate = YES;
	sprites[beer->sprite_idx].oam.doublesize = YES;
	sprites[beer->sprite_idx].oam.hflip = 0;
	sprites[beer->sprite_idx].oam.rotindex = beer->beer.rotation_idx;
	sprite_oam_rotate(beer->beer.rotation_idx, 0);
	// Offset beer due to rotation
	beer->xf += IFIX4(-4); // beer size x / 2
    beer->yf += IFIX4(-8); // beer size y / 2
	ENTITY_OAM_SYNC(beer);
}

void			beer_player_catch(t_entity *beer)
{
	game_state_score_add(GAME_BEER_CATCH_SCORE_INCREASE);
	beer_delete(beer);
}

//---------------------------------------------------------------------
