//---------------------------------------------------------------------
// SPACE TAPPER: game_entity_customers.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Entity - Customers handling
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

const ts_customer_def customers_def[CUSTOMER_TYPE_MAX] = 
{
	{ 0,  6, 512*8, IFIX4(-3), IFIX4(-3),  100,  500,  (void *)alien1_Palette,	(void *)alien1_Bitmap,	(ts_anims_set *)data_alien1_anim_set	},
	{ 1,  7, 512*8, IFIX4(-3), IFIX4(-4),  100,  500,  (void *)alien2_Palette,	(void *)alien2_Bitmap,	(ts_anims_set *)data_alien2_anim_set	},
	{ 2,  8, 512*9, IFIX4(-2), IFIX4(-5),  100,  500,  (void *)alien3_Palette,	(void *)alien3_Bitmap,	(ts_anims_set *)data_alien3_anim_set	},
	{ 3,  9, 512*11,IFIX4(-2), IFIX4(-4),  100,  600,  (void *)alien5_Palette,	(void *)alien5_Bitmap,	(ts_anims_set *)data_alien5_anim_set	},
	{ 4, 10, 512*9, IFIX4(-2), IFIX4(-4),  100,  500,  (void *)alien6_Palette,	(void *)alien6_Bitmap,	(ts_anims_set *)data_alien6_anim_set	},
	{ 5, 11, 512*13,IFIX4(-3), IFIX4(-3),  100, 1000,  (void *)alien7_Palette,	(void *)alien7_Bitmap,	(ts_anims_set *)data_alien7_anim_set	},
	{ 6, 12, 512*9, IFIX4(-3), IFIX4(-3),  100,  500,  (void *)alien8_Palette,  (void *)alien8_Bitmap,  (ts_anims_set *)data_alien8_anim_set	}
};

u16             customers_tiles_start[CUSTOMER_TYPE_MAX];

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			customers_init_videodata(void)
{
	int			i;
	
	// Copy Customers Palette & Tiles
	// FIXME: use VRAM manager
	for (i = 0; i < CUSTOMER_TYPE_MAX; i++)
	{
	    const ts_level *level = game.state.level;
	    if (level->aliens_used & (1 << i))
	    {
		    ts_customer_def *def = (ts_customer_def *)&customers_def[i];
		    palette_copy_16(def->palette_data, MEM_PALx_OBJ(def->palette_idx));
		    customers_tiles_start[i] = vram_obj_copy(def->tiles_data, def->tiles_data_size);
		}
		else
		{
		    customers_tiles_start[i] = 0;
		}
	}
}

t_entity *		customer_new(int bar_idx, int customer_type)
{
	// Create entity and add to bar list
	t_entity *c = entity_new(ENTITY_TYPE_CUSTOMER, bar_idx);
	
	// Setup entity
	c->active = YES;
	c->sub_type = customer_type;
	c->state = CUSTOMER_STATE_NORMAL;
	c->sub_state = 0;
	c->direction = bars[bar_idx].direction;
	c->xf = bars[bar_idx].xf2;
	c->yf = bars[bar_idx].yf + IFIX4(CUSTOMER_SPR_OFFSET_BAR_Y);
	c->customer.amount_drunk = 0;
	c->customer.thirstyness = (customer_type == CUSTOMER_TYPE_HEAVY_DRINKER) ? CUSTOMER_HEAVY_DRINKER_THIRSTYNESS : CUSTOMER_THIRSTYNESS;
	c->customer.move_step_delay = 1;
	c->customer.move_step_left = 50 + (rand() % 30); // FIXME: avoid using rand()
	c->customer.beer = NULL;

	// Setup Sprite
	ts_customer_def *def = (ts_customer_def *)&customers_def[customer_type];
	c->sprite_idx = sprite_new(ENTITY_SUB_PRIORITY_CUSTOMER(bar_idx));
	sprites[c->sprite_idx].oam.attr0 = SPR_A0_COLOR_16 | SPR_A0_SHAPE_SQUARE;
	sprites[c->sprite_idx].oam.attr1 = SPR_A1_SIZE_32 | (c->direction == DIRECTION_RIGHT ? SPR_A1_FLIP_H : SPR_A1_FLIP_NONE);
	sprites[c->sprite_idx].oam.attr2 = SPR_A2_PRIORITY_Set(ENTITY_SPR_PRIORITY(bar_idx)) | SPR_A2_PALETTE_Set(def->palette_idx);
	
	// Setup Animation
	animator_init(&c->anim, def->anims_set, &sprites[c->sprite_idx],
				  ANIMATOR_MODE_OAM_CHANGE_IDX, (void *)(u32)customers_tiles_start[def->type], NULL);
	animator_start(&c->anim, CUSTOMER_ANIM_STANDING, 0);
	
	// Offset sprite X position depending on direction
	// (need to be done now, as we need to have the bounding box for that)
	t_boundingbox *bb = animator_get_current_boundingbox(&c->anim) + DIRECTION_TO_OFFSET(c->direction);
	c->xf -= (c->direction == DIRECTION_LEFT) ? bb->xf1 : bb->xf2;
	ENTITY_OAM_SYNC(c);
	
	return (c);
}

void			customer_delete(t_entity *c)
{
	if (c->customer.beer)
		beer_delete(c->customer.beer);
	sprite_delete(c->sprite_idx);
	entity_delete(c);
}

void			customer_update(t_entity *c)
{
	int			dir_left = (c->direction == DIRECTION_LEFT);

	animator_update(&c->anim);
	
	switch (c->state)
	{
		case CUSTOMER_STATE_NORMAL:
		{
			if (c->customer.move_step_left == 0)
			{
	    		if (c->customer.move_step_delay-- == 0)
    				c->customer.move_step_left = 30 + (rand() % 30); // FIXME: avoid using rand()
			}
			else
			//if (c->customer.move_step_left > 0)
			{
				if (--c->customer.move_step_left == 0)
				{
					c->customer.move_step_delay = 140 + (rand() % 40); // FIXME: avoid using rand()
				}
				else
				{
					// Move (walk)
					c->xf += (dir_left ? -CUSTOMER_SPEED_WALK : CUSTOMER_SPEED_WALK);
					
					// Check if reaching end of bar
					t_boundingbox *bb_customer = animator_get_current_boundingbox(&c->anim) + DIRECTION_TO_OFFSET(c->direction);
					int d = boundingbox_cmpx_o(bb_customer, c->xf, bars[c->bar_idx].xf1 + (dir_left ? BAR_CURVED_PART_LENGTH : -BAR_CURVED_PART_LENGTH));
					if ((dir_left && d <= 0) || (!dir_left && d >= 0))
	    				player_state_slide(c->bar_idx);
		    		ENTITY_OAM_SYNC_X(c);
				}
			}
			break;
		}
		case CUSTOMER_STATE_DRINKING:
		{
		    switch (c->sub_state)
		    {
		        case CUSTOMER_SUB_STATE_DRINKING_MOVE_BACK: // Moving Back
		        {
					t_boundingbox *bb_customer = animator_get_current_boundingbox(&c->anim) + DIRECTION_TO_OFFSET(c->direction);
					int d = boundingbox_cmpx_o(bb_customer, c->xf, bars[c->bar_idx].xf2);

		        	// Delete the customer when he is out of the screen
           			if (c->customer.amount_drunk >= c->customer.thirstyness)
           			{
						if ((dir_left && d > 0) || (!dir_left && d < 0))
    			    	{
	    					// FIXME: score not constant ?
    	    		    	money_new(c->bar_idx, bars[c->bar_idx].xf2, customers_def[c->sub_type].money_given);
        					customer_delete(c);
	    	    			break;				
    	    			}
    	    		}
    		        if ((c->customer.amount_drunk >= c->customer.thirstyness)
    		            || ((dir_left && d < 0) || (!dir_left && d > 0)))
            		{
    	        	    // Move Customer
    	        	    fs16 speed = (dir_left ? CUSTOMER_SPEED_THROWN_BACK : -CUSTOMER_SPEED_THROWN_BACK);
    	        		c->xf += speed;
    	        		ENTITY_OAM_SYNC_X(c);
				
			        	// Move the Beer as well
    	    		    c->customer.beer->xf += speed;
    	    		    ENTITY_OAM_SYNC_X(c->customer.beer);
    			        if (--c->customer.move_step_left == 0)
    			            c->sub_state = CUSTOMER_SUB_STATE_DRINKING_ANIM_INIT;
            		}
    	        	else
    			    {
    			        c->sub_state = CUSTOMER_SUB_STATE_DRINKING_ANIM_INIT;
    			    }
		            break;
		        }
		        case CUSTOMER_SUB_STATE_DRINKING_ANIM_INIT: // Drinking Animation Start
		        {
					animator_start(&c->anim, CUSTOMER_ANIM_DRINKING, 0);
					animator_start(&c->customer.beer->anim, BEER_ANIM_DRINKING(c->sub_type), 0);
					c->sub_state = CUSTOMER_SUB_STATE_DRINKING_ANIM;
					break;
				}
                case CUSTOMER_SUB_STATE_DRINKING_ANIM: // Check if Drinking animation is over
                {
		        	if (c->anim.state.loop_count == 0)
			            break;

		        	// Customer release and return back the beer
		        	switch (c->sub_type)
		        	{
		        		case CUSTOMER_TYPE_FAST_THROWER:
                            customer_fast_thrower_throw(c);
		        			// beer_customer_return(c->customer.beer, BEER_SPEED_RETURNED_FAST);
		        			break;
		        		case CUSTOMER_TYPE_HEAVY_DRINKER:
		        		    customer_heavy_drinker_throw(c);
		        			beer_customer_heavy_drinker_throw_back(c->customer.beer);
            		        c->customer.beer = NULL;
		        			break;
		        		default:
					        customer_normal(c);
		        			beer_customer_return(c->customer.beer, BEER_SPEED_RETURNED_NORMAL);
            		        c->customer.beer = NULL;
		        			break;
		        		}
           		
                    break;
                }
		    }
			break;
		}
		// For fast-thrower and heavy-drinker
        case CUSTOMER_STATE_THROWING:
        {
        	if (c->anim.state.loop_count == 0)
	            break;

            // Only perform the beer throw at this moment
       	    if (c->sub_type == CUSTOMER_TYPE_FAST_THROWER)
       	    {
     			beer_customer_return(c->customer.beer, BEER_SPEED_RETURNED_FAST);
     			c->customer.beer = NULL;
	            c->state = CUSTOMER_STATE_NORMAL;
	            sprite_priority_set(c->sprite_idx, ENTITY_SUB_PRIORITY_CUSTOMER(c->bar_idx));
	            animator_start(&c->anim, CUSTOMER_ANIM_FAST_THROWER_THROWING_2, 0);
     		}
     		else
     		{
	            // Get back to normal state after throwing the beer back
                customer_normal(c);
            }
            break;
        }
	}
	
}

//---------------------------------------------------------------------

void			customer_normal(t_entity *c)
{
	c->state = CUSTOMER_STATE_NORMAL;
	sprite_priority_set(c->sprite_idx, ENTITY_SUB_PRIORITY_CUSTOMER(c->bar_idx));
	if (c->sub_type == CUSTOMER_TYPE_HEAVY_DRINKER && (c->customer.amount_drunk + 1 >= c->customer.thirstyness))
	    animator_start(&c->anim, CUSTOMER_ANIM_HEAVY_DRINKER_DRANK, 0);
	else
	    animator_start(&c->anim, CUSTOMER_ANIM_STANDING, 0);
}

void			customer_drink(t_entity *c, t_entity *beer)
{
	c->state = CUSTOMER_STATE_DRINKING;
	c->sub_state = CUSTOMER_SUB_STATE_DRINKING_MOVE_BACK;
	c->customer.beer = beer;
    c->customer.amount_drunk++;
    game_state_score_add(customers_def[c->sub_type].drink_point_given);
	sprite_priority_set(c->sprite_idx, ENTITY_SUB_PRIORITY_CUSTOMER_DRINKING(c->bar_idx));

    // Setup number of steps to go back
   	c->customer.move_step_left = (c->sub_type == CUSTOMER_TYPE_HEAVY_DRINKER) ? CUSTOMER_HEAVY_DRINKER_STEPS_BACK : CUSTOMER_STEPS_BACK;
   	if (c->customer.amount_drunk >= c->customer.thirstyness)
   	{
		t_boundingbox *bb_customer = animator_get_current_boundingbox(&c->anim) + DIRECTION_TO_OFFSET(c->direction);
		if (bars[c->bar_idx].direction == DIRECTION_LEFT)
		{
			if (boundingbox_cmpx_o(bb_customer, c->xf, bars[c->bar_idx].xf2 - IFIX4(50 + 8 + 8)) > 0)
    			c->customer.move_step_left = 255;
		}
		else
		{
			if (boundingbox_cmpx_o(bb_customer, c->xf, bars[c->bar_idx].xf2 + IFIX4(50 + 8 + 8)) < 0)
    			c->customer.move_step_left = 255;
		}
   	}
}

void			customer_watch(t_entity *c)
{
	c->state = CUSTOMER_STATE_WATCHING;
	c->sub_state = 0;
	animator_start(&c->anim, CUSTOMER_ANIM_WATCHING, 0);
	sprite_priority_set(c->sprite_idx, ENTITY_SUB_PRIORITY_CUSTOMER_WATCHING(c->bar_idx));
}

void                customer_fast_thrower_throw(t_entity *c)
{
    c->state = CUSTOMER_STATE_THROWING;
	sprite_priority_set(c->sprite_idx, ENTITY_SUB_PRIORITY_CUSTOMER(c->bar_idx));
    animator_start(&c->anim, CUSTOMER_ANIM_FAST_THROWER_THROWING_1, 0);
}

void            customer_heavy_drinker_throw(t_entity *c)
{
    c->state = CUSTOMER_STATE_THROWING;
	sprite_priority_set(c->sprite_idx, ENTITY_SUB_PRIORITY_CUSTOMER(c->bar_idx));
    animator_start(&c->anim, CUSTOMER_ANIM_HEAVY_DRINKER_THROWING, 0);
}

//---------------------------------------------------------------------
