//---------------------------------------------------------------------
// SPACE TAPPER: game_entity_money.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Entity - Money handling
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int      		money_tiles_start;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			money_init_videodata(void)
{
	palette_copy_16(game.state.level->layout->money_palette, MEM_PALx_OBJ(MONEY_PALETTE_IDX));
    money_tiles_start = vram_obj_copy(game.state.level->layout->money_tiles, 1 * 1 * TILE_SIZE_16 * 9 /* FIXME */);
}

t_entity *		money_new(int bar_idx, fs16 xf, u16 amount)
{
	// Create entity and add to bar list
	t_entity *money = entity_new(ENTITY_TYPE_MONEY, bar_idx);

	// Setup Entity
	money->active = YES;
	money->sub_type = 0;
	money->xf = xf + ((bars[bar_idx].direction == DIRECTION_LEFT) ? IFIX4(MONEY_SPR_OFFSET_BAR_X_L) : IFIX4(MONEY_SPR_OFFSET_BAR_X_R));
	money->yf = bars[bar_idx].yf + IFIX4(MONEY_SPR_OFFSET_BAR_Y);
	// money->direction = bars[bar_idx].direction;
	money->state = MONEY_STATE_NORMAL;
	money->sub_state = 0;
	money->money.amount = amount;
	money->money.life_left = MONEY_DEFAULT_TIMEOUT;

	// Setup Sprite
	money->sprite_idx = sprite_new(ENTITY_SUB_PRIORITY_MONEY(bar_idx));
	sprites[money->sprite_idx].oam.attr0 = SPR_A0_COLOR_16 | SPR_A0_SHAPE_SQUARE;
	sprites[money->sprite_idx].oam.attr1 = SPR_A1_SIZE_8;
	sprites[money->sprite_idx].oam.attr2 = SPR_A2_PRIORITY_Set(ENTITY_SPR_PRIORITY(bar_idx+1)) | SPR_A2_PALETTE_Set(MONEY_PALETTE_IDX) | SPR_A2_CHAR_Set(money_tiles_start);
	ENTITY_OAM_SYNC(money);

	// Setup Animation
	animator_init(&money->anim, (ts_anims_set *)data_money_anim_set, &sprites[money->sprite_idx],
				  ANIMATOR_MODE_OAM_CHANGE_IDX, (void *)money_tiles_start, NULL);
	animator_start(&money->anim, MONEY_ANIM_SHINING, 0);

	return (money);
}

void			money_delete(t_entity *money)
{
	sprite_delete(money->sprite_idx);
	entity_delete(money);
}

void			money_update(t_entity *money)
{
	animator_update(&money->anim);
    switch (money->state)
    {
        case MONEY_STATE_NORMAL: // Staying on the bar and Shining
        {
       		// Check if player center is colliding with the money
        	if (money->bar_idx == player.state->bar_idx)
        	{
        		// Create money bounding box
				t_boundingbox bb_money;
				bb_money.xf1 = money->xf + IFIX4(0-3); // IFIX4(bb_money->x1);
				bb_money.xf2 = money->xf + IFIX4(7+3); // IFIX4(bb_money->x2);

				// Now perform the actual collision
				t_boundingbox *bb_player = animator_get_current_boundingbox(&player.anim) + DIRECTION_TO_OFFSET(player.state->direction);
        		if (boundingboxes_overlap_center_o(&bb_money, bb_player, player.state->xf, 0))
					//if (money->xf + IFIX4(8) > player.state->xf && money->xf < player.state->xf + IFIX4(24) /* FIXME */)
    			{
					game_state_score_add(money->money.amount);
					money_delete(money);
		    	}
		    }
		    
		    // Update money life
           	if (--money->money.life_left == 0)
			{
				animator_start(&money->anim, MONEY_ANIM_VANISHING, 0);
				money->state = MONEY_STATE_VANISHING;
				return;
			}
			break;
		}
        case MONEY_STATE_VANISHING: // Vanishing
        {
			if (money->anim.state.loop_count > 0)
			{
            	money_delete(money);
				return;
			}
			break;
		}
    }
}

//---------------------------------------------------------------------
