//---------------------------------------------------------------------
// SPACE TAPPER: game_player.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Player
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

t_player	player;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FIXME: priority computation

void		player_init_videodata(void)
{
	palette_copy_16((u16 *)bartender_Palette, MEM_PALx_OBJ(PLAYER_PALETTE_IDX));
	/* PLAYER_TILES_START = */ vram_obj_alloc(32 * TILE_SIZE_16);
}

void		player_init(void)
{
	// Setup Player Attributes (State)
	player.state = &player.states_stack[0];
	player.state->active = YES;
	player.state->state = PLAYER_STATE_IDLING;
	player.state->sub_state = 0;
	player.state->state_custom_value = 0;
	player.state->waiting_anim_loop = NO;
	player.state->bar_idx = PLAYER_BAR_START;
	player.state->direction = bars[player.state->bar_idx].direction;
	player.state->xf = bars[player.state->bar_idx].xf_player;
	player.state->yf = FADDI(bars[player.state->bar_idx].yf_player, PLAYER_SPR_OFFSET_Y, 4);
	player.state->xi = FINT(player.state->xf, 4);
	player.state->yi = FINT(player.state->yf, 4);

	// Setup Player Attributes (Global)
	player.beer = NULL;
	player.beer_catched = NULL;
	
	// Setup Player Sprite
	player.sprite_idx = sprite_new(ENTITY_SUB_PRIORITY_PLAYER(player.state->bar_idx));
	player.sprite = &sprites[player.sprite_idx];
	player.sprite->oam.attr0 = SPR_A0_COLOR_16 | SPR_A0_SHAPE_TALL | SPR_A0_Y_Set(player.state->yi);
	player.sprite->oam.attr1 = SPR_A1_SIZE_64 | SPR_A1_X_Set(player.state->xi) | (player.state->direction ? SPR_A1_FLIP_H : SPR_A1_FLIP_NONE);
	player.sprite->oam.attr2 = SPR_A2_PRIORITY_Set(2) | SPR_A2_CHAR_Set(0) | SPR_A2_PALETTE_Set(PLAYER_PALETTE_IDX);
	
	// Setup Player Animation
	animator_init(&player.anim, (ts_anims_set *)data_bartender_anim_set, player.sprite,
				  ANIMATOR_MODE_VRAM_COPY_DATA, (u16 *)bartender_Bitmap, MEM_VRAM_OBJ_TILEx(0));
	animator_start(&player.anim, PLAYER_ANIM_STANDING, 0);
}

void		player_delete(void)
{
	sprite_delete(player.sprite_idx);
}

void		player_update(void)
{
	// Update animation & sprites
	animator_update(&player.anim);
	
	// Return if we're waiting for end of animation
	if (player.state->waiting_anim_loop == YES)
	{
		if (player.anim.state.loop_count == 0)
			return;
		player.anim.state.loop_count = 0;
		player.state->waiting_anim_loop = NO;
	}
	
	// Movements ------------------------------------------------------------

	// Left/Right Running ---------------------------------------------------
	if (player.state->state & (PLAYER_STATE_IDLING | PLAYER_STATE_RUNNING | PLAYER_STATE_SERVING))
	{
		fs16	vel_xf;
		fs16	min_xf, max_xf;
		int		direction_old;
		
		// Set velocity depending on player's action
			 if (game.keys & REG_KEYINPUT_KEY_RIGHT)	vel_xf = +PLAYER_SPEED;
		else if (game.keys & REG_KEYINPUT_KEY_LEFT)		vel_xf = -PLAYER_SPEED;
		else											vel_xf =  0;

		if (vel_xf != 0)
		{
			// Update player direction accordingly
			// Note: direction is intentionnally updated, to allow changing direction when you're on the limit
			direction_old = player.state->direction;
			player.sprite->oam.hflip = player.state->direction = (vel_xf > 0) ? DIRECTION_RIGHT : DIRECTION_LEFT;

			// Limit movements to left and right bounds
			if (bars[player.state->bar_idx].direction == DIRECTION_LEFT)
			{
				min_xf = (bars[player.state->bar_idx].xf_player - player.state->xf);
				max_xf = (bars[player.state->bar_idx].xf2 - IFIX4(PLAYER_SPR_SIZE_X)) - player.state->xf;
			}
			else
			{
				min_xf = (bars[player.state->bar_idx].xf2 - player.state->xf);
				max_xf = (bars[player.state->bar_idx].xf_player - player.state->xf);
			}
			if (vel_xf < min_xf)
				vel_xf = min_xf;
			if (vel_xf > max_xf)
				vel_xf = max_xf;
		}
		
		if (vel_xf != 0)
		{
			// Perform the actual move
			// Cancel beer being served if there's one and we're attempting to move
			if (player.beer != NULL)
				player_beer_delete();
			
			// Move player X position and updated integer value
			player.state->xf += vel_xf;
			player.state->xi = FINT(player.state->xf, 4);
			
			// Switch to Running state if player was previously not Running
			if (player.state->state != PLAYER_STATE_RUNNING)
			{
				player.state->state = PLAYER_STATE_RUNNING;
				if (direction_old != player.state->direction)
					animator_start(&player.anim, PLAYER_ANIM_RUNNING_SWITCH, 0);
				else
					animator_start(&player.anim, PLAYER_ANIM_RUNNING, 0);
			}
		}
		else /* vel_xf == 0 */
		{
			// Switch to Idling state if player was previously Running
			if (player.state->state == PLAYER_STATE_RUNNING)
			{
				player.state->state = PLAYER_STATE_IDLING;
				animator_start(&player.anim, PLAYER_ANIM_STANDING, 0);
			}
		}
	}

	// Up/Down Switching ----------------------------------------------------
	if (player.state->state & (PLAYER_STATE_IDLING | PLAYER_STATE_RUNNING | PLAYER_STATE_SERVING | PLAYER_STATE_BEER_CATCHING))
	{
		if ((game.keys & REG_KEYINPUT_KEY_DOWN)) // && !(game.keys_old & REG_KEYINPUT_KEY_DOWN))
		{
			// Cancel beer being served, count beer being catched
			if (player.beer != NULL)
				player_beer_delete();
			if (player.beer_catched != NULL)
				player_beer_catched_catch();
			player.state->state = PLAYER_STATE_SWITCHING_DOWN;
			player.state->waiting_anim_loop = YES;
			animator_start(&player.anim, PLAYER_ANIM_SWITCHING, 0);
		}
		else
		if ((game.keys & REG_KEYINPUT_KEY_UP)) // && !(game.keys_old & REG_KEYINPUT_KEY_UP))
		{
			// Cancel beer being served, count beer being catched
			if (player.beer != NULL)
				player_beer_delete();
			if (player.beer_catched != NULL)
				player_beer_catched_catch();
			player.state->state = PLAYER_STATE_SWITCHING_UP;
			player.state->waiting_anim_loop = YES;
			animator_start(&player.anim, PLAYER_ANIM_SWITCHING, 0);
		}
	}
	else
	if (player.state->state & (PLAYER_STATE_SWITCHING_DOWN | PLAYER_STATE_SWITCHING_UP))
	{
		if (player.state->state & PLAYER_STATE_SWITCHING_DOWN)
		{
			if (++player.state->bar_idx == BAR_MAX)
				player.state->bar_idx = 0;
		}
		else
		{
			if (player.state->bar_idx-- == 0)
				player.state->bar_idx = BAR_MAX - 1;
		}
		sprite_priority_set(player.sprite_idx, ENTITY_SUB_PRIORITY_PLAYER(player.state->bar_idx));
		bars_entities_update_priority();
		player.state->xf = bars[player.state->bar_idx].xf_player;
		player.state->yf = FADDI(bars[player.state->bar_idx].yf_player, PLAYER_SPR_OFFSET_Y, 4);
		player.state->xi = FINT(player.state->xf, 4);
		player.state->yi = FINT(player.state->yf, 4);
		player.sprite->oam.hflip = player.state->direction = bars[player.state->bar_idx].direction;
		player.state->state = PLAYER_STATE_IDLING;
	}	

	// ..
#if 0
	if (game.keys & REG_KEYINPUT_KEY_B)
		if (!(game.keys_old & REG_KEYINPUT_KEY_B))
			player_state_slide(2);
#endif

	// Victory --------------------------------------------------------------
	if (player.state->state & (PLAYER_STATE_VICTORY))
	{
		player_victory_next_level();
	}

	// Sliding --------------------------------------------------------------
	if (player.state->state & (PLAYER_STATE_SLIDING))
	{
		switch (player.state->sub_state)
		{
			case PLAYER_SUB_STATE_SLIDING_SLIDING:
			{
				int dir_left = player.state->direction == DIRECTION_LEFT;
				// Move (Slide)
				player.state->xf += dir_left ? PLAYER_SLIDING_SPEED : -PLAYER_SLIDING_SPEED;
				player.state->xi = FINT(player.state->xf, 4);
				
				// Check if reaching end of bar
				t_boundingbox *bb = animator_get_current_boundingbox(&player.anim) + DIRECTION_TO_OFFSET(player.state->direction);
				int d = boundingbox_cmpx_o(bb, player.state->xf, bars[player.state->bar_idx].xf2);
				if ((dir_left && d > 0) || (!dir_left && d < 0))
				{
					player.state->sub_state = PLAYER_SUB_STATE_SLIDING_CRASHING;
					player.state->state_custom_value = 0;
				}
				break;
			}
			case PLAYER_SUB_STATE_SLIDING_CRASHING:
			{
				int step = ++player.state->state_custom_value;
				if (step & 3)
					break;
				step /= 4;
				if (step == CRASH_STEP_MAX)
				{
					player.state->sub_state = PLAYER_SUB_STATE_SLIDING_DELAY;
					player.state->state_custom_value = 0;
				}
				else
				{
					crash_do_step(step);
				}
				break;
			}
			case PLAYER_SUB_STATE_SLIDING_DELAY:
			{
				if (++player.state->state_custom_value == PLAYER_SLIDING_DELAY)
				{
				    player_live_lose();
				}
				break;
			}
		}
	}

	// Beer Catching --------------------------------------------------------
	if (player.state->state & (PLAYER_STATE_BEER_CATCHING))
	{
		if (player.anim.state.loop_count > 0)
		{
			// We are reaching this point if the catching animation is done
			player_beer_catched_catch();
		}
	}
	
	// Beer Breaking --------------------------------------------------------
	if (player.state->state & (PLAYER_STATE_BEER_BREAKING))
	{
	    // Nothing to do
	}

	// Tapping/Serving ------------------------------------------------------
	if (player.state->state & (PLAYER_STATE_IDLING | PLAYER_STATE_RUNNING | PLAYER_STATE_SERVING))
	{
		if (player.beer || !(game.keys & (REG_KEYINPUT_KEY_LEFT | REG_KEYINPUT_KEY_RIGHT)))
		{
			if ((game.keys & REG_KEYINPUT_KEY_A) /* && !(game.keys & REG_KEYINPUT_KEY_B) */)
			{
				player.state->xf = bars[player.state->bar_idx].xf_player;
				player.state->xi = FINT(player.state->xf, 4);
				player.sprite->oam.hflip = player.state->direction = bars[player.state->bar_idx].direction;
        	
				// Create beer if needed
				if (player.beer == NULL)
				{
					player.beer = beer_new(player.state->bar_idx, 
										   player.state->xi + ((bars[player.state->bar_idx].direction == DIRECTION_LEFT) ? BEER_SPR_OFFSET_PLAYER_X : PLAYER_SPR_SIZE_X - BEER_SPR_OFFSET_PLAYER_X - BEER_SPR_SIZE_X), // FIXME: ...
										   player.state->yi + BEER_SPR_OFFSET_PLAYER_Y);
					player.state->state = PLAYER_STATE_SERVING;
					animator_start(&player.anim, PLAYER_ANIM_SERVING, 0);
					tapper_down(player.state->bar_idx);
				}
				else
					// Else restart serving animation
					if (!(game.keys_old & REG_KEYINPUT_KEY_A))
					{
						animator_start(&player.anim, PLAYER_ANIM_SERVING_SERVING, 0);
						tapper_down_immediate(player.state->bar_idx);
					}
        	    
				// Fill beer
				if (player.beer->beer.filled < BEER_FILLED_MAX)
				{
					// FIXME: update beer animation
					player.beer->beer.filled++;
					int beer_anim_step;
					if (player.beer->beer.filled < 8) // FIXME : should play an animation
						beer_anim_step = BEER_ANIM_STEP(0);
					else					
					{
						// FIXME: avoid divisions!!
						beer_anim_step = BEER_ANIM_STEP(((player.beer->beer.filled - 8) * BEER_ANIM_STEP_MAX) / (BEER_FILLED_MAX - 8));
					}
					// FIXME: Avoid updating the animation frame everytime
					//if (beer_anim_step != player.beer->anim.state.frame_current)
						animator_start(&player.beer->anim, beer_anim_step, 0);
					if (player.beer->beer.filled == BEER_FILLED_MAX)
					{
						animator_start(&player.anim, PLAYER_ANIM_SERVING_HOLD, 0);
						tapper_up(player.state->bar_idx);
					}
				}
			}
			else
			if (game.keys_old & REG_KEYINPUT_KEY_A)
			{
				if (player.beer)
				{
				    // Set tapper back to Up position
					tapper_up(player.state->bar_idx);
        	
					if (player.beer->beer.filled == BEER_FILLED_MAX)
					{
						// Throw beer
						beer_player_throw(player.beer);
												
						// Detach from player
						player.beer = NULL;
					
						// Go back to idling state
						player.state->state = PLAYER_STATE_IDLING;
						player.state->waiting_anim_loop = YES;
						animator_start(&player.anim, PLAYER_ANIM_THROWING, 0);
					}
					else
					{
						animator_start(&player.anim, PLAYER_ANIM_SERVING_HOLD, 0);
					}
				}
			}
		}
	}

	// Update Player Sprite data in shadow OAM ------------------------
	player.sprite->oam.y = player.state->yi;
	player.sprite->oam.x = player.state->xi;
}

//---------------------------------------------------------------------

void		player_state_beer_catching(t_entity *beer)
{
	// Setup pointer to beer being catched
	player.beer_catched = beer;

	// If catching while serving, and in 'down' state, get back to 'up' state
	if (player.beer != NULL)
 	{
		animator_set(&player.anim, PLAYER_ANIM_SERVING_HOLD, 0);
        tapper_up(player.state->bar_idx);
    }

	// Save previous state
	player_state_push();

	// Move player
	player.state->xf = bars[player.state->bar_idx].xf_player;
	player.sprite->oam.x = player.state->xi = FINT4(player.state->xf);

	// Setup sprite direction
	player.sprite->oam.hflip = player.state->direction = bars[player.state->bar_idx].direction;

	// Setup catching state
	player.state->state = PLAYER_STATE_BEER_CATCHING;

	// Catching while serving
	// (also see code at the beginning of the function)
	if (player.beer != NULL)
	{
		player.state->sub_state = PLAYER_SUB_STATE_BEER_CATCHING_SERVING;
		player.state->waiting_anim_loop = NO;
	    animator_start(&player.anim, PLAYER_ANIM_BEER_CATCHING_SERVING_UP, 0);
	}
	else
	// Catching without serving
	{
		player.state->sub_state = PLAYER_SUB_STATE_BEER_CATCHING_IDLING;
		player.state->waiting_anim_loop = NO;
	    animator_start(&player.anim, PLAYER_ANIM_BEER_CATCHING_IDLING, 0);
	}
}

void		player_beer_delete(void)
{
	beer_delete(player.beer);
	player.beer = NULL;
	tapper_up(player.state->bar_idx);
}

void		player_beer_catched_delete(void)
{
   	beer_delete(player.beer_catched);
   	player.beer_catched = NULL;
	player_state_pop();
}

void		player_beer_catched_catch(void)
{
	beer_player_catch(player.beer_catched);
	player.beer_catched = NULL;
	player_state_pop();
}

//---------------------------------------------------------------------

void		player_state_slide(int bar_idx)
{
    // Remove beers if player is serving and/or catching one
	if (player.beer != NULL)
		player_beer_delete();
    if (player.beer_catched != NULL)
    	player_beer_catched_delete();

    // Set player state
	player.state->state = PLAYER_STATE_SLIDING;
	player.state->sub_state = PLAYER_SUB_STATE_SLIDING_SLIDING;
	
	// Move player to bar where he slide
	player.state->bar_idx = bar_idx;
	sprite_priority_set(player.sprite_idx, ENTITY_SUB_PRIORITY_PLAYER(bar_idx));
	bars_entities_update_priority();

	player.state->yf = bars[player.state->bar_idx].yf + IFIX4(PLAYER_SPR_OFFSET_Y);
	if (bars[player.state->bar_idx].direction == DIRECTION_LEFT)
		player.state->xf = bars[player.state->bar_idx].xf1 + IFIX4(8);
	else
		player.state->xf = bars[player.state->bar_idx].xf1 - IFIX4(8) - IFIX4(PLAYER_SPR_SLIDING_SIZE_X);
	player.sprite->oam.y = player.state->yi = FINT(player.state->yf, 4);
	player.sprite->oam.x = player.state->xi = FINT(player.state->xf, 4);
	
	// Update sprite attributes with right direction and horizontal sprite shape
	player.sprite->oam.hflip = player.state->direction = bars[player.state->bar_idx].direction;
	player.sprite->oam.shape = SPR_SHAPE_WIDE;
	
	// Play sliding animation
	animator_start(&player.anim, PLAYER_ANIM_SLIDING, 0);
	game.update_flags &= ~GAME_UPDATE_ENTITIES;
}

void        player_state_beer_breaking(int bar_idx)
{
    // Remove beers if player is serving and/or catching one
	if (player.beer != NULL)
		player_beer_delete();
    if (player.beer_catched != NULL)
    	player_beer_catched_delete();

    // Set player state
    player.state->state = PLAYER_STATE_BEER_BREAKING;

 	// Update sprite attributes with right direction
	player.sprite->oam.hflip = player.state->direction = bars[player.state->bar_idx].direction;
   
    // Play Beer Breaking animation
    animator_start(&player.anim, PLAYER_ANIM_BEER_BREAKING, 0);
}

//---------------------------------------------------------------------

void        player_live_lose(void)
{
    game.state.lives--;
    game_main_restart();
}

void		player_victory(void)
{
    // Remove beer being served, count beer being catched
	if (player.beer != NULL)
		player_beer_delete();
    if (player.beer_catched != NULL)
    	player_beer_catched_catch();

    // Set player state
    player.state->state = PLAYER_STATE_VICTORY;
    player.state->waiting_anim_loop = YES;
	
 	// Update sprite attributes with right direction
	player.sprite->oam.hflip = player.state->direction = DIRECTION_OPPOSITE(bars[player.state->bar_idx].direction);
	
	// Start animation
    animator_start(&player.anim, PLAYER_ANIM_VICTORY, 0);
}

void		player_victory_next_level(void)
{
	game_main_next_level();
}

//---------------------------------------------------------------------

void	player_state_push(void)
{
	if (player.state + 1 > player.states_stack + sizeof (player.states_stack))
		fatal("player stack overflow\n");
	animator_state_backup(&player.anim, &player.state->anim_state);
	player.state++;
	CPU_Naive_Memory_Copy(player.state, player.state - 1, sizeof (*player.state));
}

void	player_state_pop(void)
{
	if (player.state - 1 < player.states_stack)
		fatal("player stack underflow\n");
	player.state--;
	animator_state_restore(&player.anim, &player.state->anim_state);

	// Update Player Sprite data in shadow OAM ------------------------
	player.sprite->oam.y = player.state->yi;
	player.sprite->oam.x = player.state->xi;
	player.sprite->oam.hflip = player.state->direction;
}

//---------------------------------------------------------------------
