//---------------------------------------------------------------------
// SPACE TAPPER: game_special_entertainers.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Special - Entertainers
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

typedef struct
{
	t_bg_block			block;
	t_animator			anim;
	t_bg_block_map      block_platform;
	int					state;
	int					sub_state;
	int					play_length;
}						t_entertainers;

static t_entertainers 	entertainers;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void            	entertainers_init_videodata(void)
{
	// Allocate video memory and create block
	vram_bg_copy_block_256((u16 *)entertainers_Bitmap, &entertainers.block, 9, 6);
	if (game.state.level->special == GAME_SPECIAL_ENTERTAINERS_2)
		vram_bg_copy_block_map_256(&entertainers.block_platform, (u16 *)l3_layout_platform_Tiles, sizeof (l3_layout_platform_Tiles), (u16 *)l3_layout_platform_Map, 10, 10);

	// Setup Animation
	animator_init(&entertainers.anim, (ts_anims_set *)data_entertainers_anim_set, NULL,
				  ANIMATOR_MODE_VRAM_COPY_DATA, (u16 *)entertainers_Bitmap, MEM_VRAM_BG_TILEx(entertainers.block.tiles_idx * 2));
}

//---------------------------------------------------------------------

void				entertainers_init(void)
{
	// Setup state
	entertainers.state = ENTERTAINERS_STATE_AWAY;
	entertainers.sub_state = 0;
}

void            	entertainers_update(void)
{
	switch (entertainers.state)
	{
		case ENTERTAINERS_STATE_AWAY:
		{
			break;
		}
		
		case ENTERTAINERS_STATE_PLATFORM_APPEARING:
		{
			if (++entertainers.sub_state == 1)
			{
				// Blit platform map to BG2
				background_block_map_draw(&game.bg2, &entertainers.block_platform, 10, 0);

				// Setup BG2 scrolling
				game.bg2.scroll_x = 0;
				game.bg2.scroll_y = 80;
				background_sync_scroll(&game.bg2);
			}
			else
			{
				// Scroll BG2
				game.bg2.scroll_y--;
				background_sync_scroll(&game.bg2);
				
				if (entertainers.sub_state == 60)
				{
					entertainers.state = ENTERTAINERS_STATE_APPEARING;
					entertainers.sub_state = 0;
				}
			}
			break;
		}

		case ENTERTAINERS_STATE_APPEARING:
		{
			if (entertainers.sub_state == 0)
			{
				entertainers.sub_state = 1;

				// Blit block to BG1
				background_block_draw(&game.bg1, &entertainers.block, 11, 0, BG_MAP_FLIP_NONE);
	
				// Start Animation
				animator_start(&entertainers.anim, ENTERTAINERS_ANIM_APPEARING, 0);
			}
			if (entertainers.anim.state.loop_count > 0)
			{
				entertainers.state = ENTERTAINERS_STATE_ACTIVE;
				entertainers.sub_state = 0;
			}
			animator_update(&entertainers.anim);
			break;
		}
		
		case ENTERTAINERS_STATE_ACTIVE:
		{
			if (entertainers.sub_state == 0)
			{
				entertainers.sub_state = 1;
				
				// Entertainers just finished to appear, set some aliens to watching state
				entertainers_aliens_watch_start();
			}
			if (--entertainers.play_length == 0)
			{
				// Set Disappearing state
				entertainers.state = ENTERTAINERS_STATE_DISAPPEARING;
				// Start Disappearing animation
				animator_start(&entertainers.anim, ENTERTAINERS_ANIM_DISAPPEARING, 0);
				return;
			}
			animator_update(&entertainers.anim);
			break;
		}
		
		case ENTERTAINERS_STATE_DISAPPEARING:
		{
			if (entertainers.anim.state.loop_count > 0)
			{
	            if (game.state.level->special == GAME_SPECIAL_ENTERTAINERS_2)
	            {
    		        entertainers.state = ENTERTAINERS_STATE_PLATFORM_DISAPPEARING; // Layout 3
                    entertainers.sub_state = 0;
    		    }
    		    else
    		    {
				    entertainers.state = ENTERTAINERS_STATE_AWAY;
				}
				
				// Clear block in BG1
				background_block_clear(&game.bg1, &entertainers.block, 11, 0, 0x0000);
				
				// Set all watching customers back to their normal state
				entertainers_aliens_watch_stop();
			}
			animator_update(&entertainers.anim);
			break;
		}
		
		case ENTERTAINERS_STATE_PLATFORM_DISAPPEARING:
		{
			if (++entertainers.sub_state < 60)
			{
			    // Scroll BG2
			    game.bg2.scroll_y++;
			    background_sync_scroll(&game.bg2);
			}
			else
			{
				// Restore back away state
				entertainers.state = ENTERTAINERS_STATE_AWAY;
			}
		    break;
		}
	}
}

void				entertainers_active(int length)
{
	// Setup State
	if (game.state.level->special == GAME_SPECIAL_ENTERTAINERS_2)
		entertainers.state = ENTERTAINERS_STATE_PLATFORM_APPEARING; // Layout 3
	else
		entertainers.state = ENTERTAINERS_STATE_APPEARING;
	entertainers.sub_state = 0;
	
	// Setup play length
	entertainers.play_length = length;
}

void        entertainers_aliens_watch_start(void)
{
    int     i;

    for (i = 0; i < BAR_MAX; i++)
	{
	    t_entity *entity = bars_entities[i];
		while (entity != NULL)
		{
		    if (entity->type == ENTITY_TYPE_CUSTOMER && entity->state == CUSTOMER_STATE_NORMAL)
			    if ((rand() & 1) == 0) // FIXME: --> should be part of game rules
				    customer_watch(entity);
		    entity = entity->next;
		}
	}
}

void        entertainers_aliens_watch_stop(void)
{
    int     i;

    for (i = 0; i < BAR_MAX; i++)
    {
        t_entity *entity = bars_entities[i];
        while (entity != NULL)
        {
            if (entity->type == ENTITY_TYPE_CUSTOMER && entity->state == CUSTOMER_STATE_WATCHING)
                customer_normal(entity);
            entity = entity->next;
        }
	}
}

//---------------------------------------------------------------------
