//---------------------------------------------------------------------
// SPACE TAPPER: game_special_tubealien.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Special - Tube Alien
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

// Tube Alien + Bubbles data
typedef struct
{
	u16					tubealien_tiles_start;
	u16					tubealien_sprite_idx;
	u16					bubbles_tiles_start;
	u16					bubbles_sprite_idx;
	t_animator			tubealien_anim;
	t_animator			bubbles_anim;
}						t_tubealien;

static t_tubealien		tubealien;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void            	tubealien_init_videodata(void)
{
	// Copy Palette	
	palette_copy_16((u16 *)l2_tubealien_Palette, MEM_PALx_OBJ(TUBEALIEN_PALETTE_IDX));

	// Allocate video memory
	tubealien.tubealien_tiles_start = vram_obj_alloc(4 * 4 * TILE_SIZE_16);
	tubealien.bubbles_tiles_start = vram_obj_alloc(2 * 2 * TILE_SIZE_16);
	
	// Setup Animations
	animator_init(&tubealien.tubealien_anim, (ts_anims_set *)data_tubealien_anim_set, NULL,
				  ANIMATOR_MODE_VRAM_COPY_DATA, (u16 *)l2_tubealien_Bitmap, MEM_VRAM_OBJ_TILEx(tubealien.tubealien_tiles_start));
	animator_init(&tubealien.bubbles_anim, (ts_anims_set *)data_tubealien_bubbles_anim_set, NULL,
				  ANIMATOR_MODE_VRAM_COPY_DATA, (u16 *)l2_tubealien_bubbles_Bitmap, MEM_VRAM_OBJ_TILEx(tubealien.bubbles_tiles_start));
}

//---------------------------------------------------------------------

void            	tubealien_init(void)
{
	// Draw block on background 1
	// background_block_draw(&game.bg1, &tubealien.block, 14, 2, BG_MAP_FLIP_NONE);

	// Setup Sprites
	t_sprite_oam *sprite_oam;
	tubealien.tubealien_sprite_idx = sprite_new(1);
	sprite_oam = &sprites[tubealien.tubealien_sprite_idx].oam;
	sprite_oam->attr0 = SPR_A0_COLOR_16 | SPR_A0_SHAPE_SQUARE | SPR_A0_Y_Set(12);
	sprite_oam->attr1 = SPR_A1_SIZE_32 | SPR_A1_X_Set(116);
	sprite_oam->attr2 = SPR_A2_PRIORITY_Set(3) | SPR_A2_CHAR_Set(tubealien.tubealien_tiles_start) | SPR_A2_PALETTE_Set(TUBEALIEN_PALETTE_IDX);

	tubealien.bubbles_sprite_idx = sprite_new(2);
	sprite_oam = &sprites[tubealien.bubbles_sprite_idx].oam;
	sprite_oam->attr0 = SPR_A0_COLOR_16 | SPR_A0_SHAPE_SQUARE | SPR_A0_Y_Set(18);
	sprite_oam->attr1 = SPR_A1_SIZE_16 | SPR_A1_X_Set(132);
	sprite_oam->attr2 = SPR_A2_PRIORITY_Set(3) | SPR_A2_CHAR_Set(tubealien.bubbles_tiles_start) | SPR_A2_PALETTE_Set(TUBEALIEN_PALETTE_IDX);

	// Start playing animation
	animator_start(&tubealien.tubealien_anim, 0, 0);
	animator_start(&tubealien.bubbles_anim, 0, 0);
}

//---------------------------------------------------------------------

void				tubealien_update(void)
{
	// Update running animation
	animator_update(&tubealien.tubealien_anim);
	animator_update(&tubealien.bubbles_anim);
}

//---------------------------------------------------------------------
