//---------------------------------------------------------------------
// GBALIB: background.c
// by Omar Cornut
//---------------------------------------------------------------------
// Background Handling System
//---------------------------------------------------------------------
// This	system is very preliminar.
// It currently	only includes basic	features needed	for	Tapper.
// Will	probably get an	overhaul once a	more complex game is done.
//---------------------------------------------------------------------

#include "gbalib.h"

//---------------------------------------------------------------------

// FUNCTION: background_init()
// Initialize a	background
void		background_init(t_bg *bg, int number, u16 cnt)
{
	bg->number = number;
	bg->cnt	= cnt;
	
	// Copy	shadowed data to hardware registers
	REG_BGxCNT(number)  = cnt;
	REG_BGxHOFS(number)	= bg->scroll_x = 0;
	REG_BGxVOFS(number)	= bg->scroll_y = 0;

	// Setup Tiles & Map pointers
	// FIXME: create and use dedicated macros
	bg->tiles		= (u16 *)((u8 *)MEM_VRAM_BG	+ (0x4000 *	REG_BGxCNT_CHAR_BASE_Get(cnt)));
	bg->map			= (u16 *)((u8 *)MEM_VRAM_BG	+ (	0x800 *	REG_BGxCNT_SCREEN_BASE_Get(cnt)));
}

// FUNCTION: background_sync_cnt()
// Synchronize shadow CNT register with	hardware
inline void		background_sync_cnt(t_bg *bg)
{
	REG_BGxCNT(bg->number) = bg->cnt;
}

// FUNCTION: background_set_priority()
// Change priority for a given background
void		background_set_priority(t_bg *bg, int priority)
{
	bg->cnt	= (bg->cnt & ~REG_BGxCNT_PRIORITY_Mask);
	bg->cnt |= REG_BGxCNT_PRIORITY_Set(priority);
	background_sync_cnt(bg);
}

// FUNCTION: background_sync_scroll()
// Synchronize shadowed	scrolling values with hardware registers
void		background_sync_scroll(t_bg	*bg)
{
	REG_BGxHOFS(bg->number)	= bg->scroll_x;
	REG_BGxVOFS(bg->number)	= bg->scroll_y;
}

// FUNCTION: background_map_clear()
// Clear background	map	with given value
// FIXME: untested with	maps of	size !=	256x256
void		background_map_clear(t_bg *bg, u16 map_value)
{
	int		i;
	u32		map_value_32 = map_value ||	(map_value << 16);
	u32	*	bg_map_32 =	(u32 *)bg->map;
	
	// Find	out	number of total	cell defining the map
	// FIXME: see above
	switch (bg->cnt	& REG_BGxCNT_SCREEN_SIZE_Mask)
	{
		case REG_BGxCNT_SCREEN_SIZE_256_256: i = 32	* 32; break;
		case REG_BGxCNT_SCREEN_SIZE_512_256: i = 64	* 32; break;
		case REG_BGxCNT_SCREEN_SIZE_256_512: i = 32	* 64; break;
		case REG_BGxCNT_SCREEN_SIZE_512_512: 
		default:							 i = 64	* 64; break;
	}
	
	// Clear map
	while (i > 0)
	{
		*bg_map_32++ = map_value_32;
		*bg_map_32++ = map_value_32;
		*bg_map_32++ = map_value_32;
		*bg_map_32++ = map_value_32;
		i -= 8;
	}
}

// FUNCTION: background_map_copy()
// Copy	given source to	background map
// FIXME: currently only copy visible area of 30x20	tiles
// FIXME: should unroll/speedup (but do we care about that right now ?)
void		background_map_copy(t_bg *bg, u16 *map, int flip)
{
	int		i;
	u16	*	src;
	u16	*	dst;
	
	if (flip & BG_MAP_FLIP_H)
	{
		src	= map;
		dst	= bg->map + RES_WIDTH_TILES - 1;
		for	(i = 0;	i <	RES_HEIGHT_TILES; i++)
		{
			int j;
			for (j = 0; j < RES_WIDTH_TILES; j++)
				*dst-- = *src++ ^ BG_MAP_FLIP_H;
			dst	+= ((256 / 8) - 1) * 2;
		}
	}
	else
	{
		src	= map;
		dst	= bg->map;
		for	(i = 0;	i <	RES_HEIGHT_TILES; i++)
		{
			DMA_Copy(3,	DMA_16_NOW,	(void *)src, (void *)dst, RES_WIDTH_TILES);
			src	+= RES_WIDTH_TILES;
			dst	+= (256	/ 8);
		}
	}
}

// FUNCTION: background_block_draw()
// Copy	given background block to background, at given position
// FIXME: bg_block sizes should	be const, bg_block source address may be usable	as const if	game engine
// behavior	is well	defined.
// FIXME: should clean the code, may want to make the whole	thing more high-level and genericly
// usable for other	functions. May want	to be able to specify clipping.	Should I improve the whole
// so a	full featured BG Renderer ?
void		background_block_draw(t_bg *bg, t_bg_block *bg_block, int dst_x, int dst_y,	int	flip)
{
	u16	*	dst;
	int		x, y;
	u16		tiles_idx;
	int		size_x,	diff_x;
	int		clip_x;

	// Calculate destination address
	dst	= bg->map +	(dst_x + (dst_y	<< 5));

	// Setup original X	size
	size_x = bg_block->size_x;

	// H-Flipped version
	if (flip & BG_MAP_FLIP_H)
	{
		// Setup starting tile index
		tiles_idx =	bg_block->tiles_idx	| BG_MAP_FLIP_H;

		// Perform X clipping
		// (Note: clipping is only done	on right & bottom
		clip_x = dst_x - size_x	+ 1;

		// dprintf("[draw] dst_x = %d, dst_y = %d, size_x =	%d\n", dst_x, dst_y, size_x);
		
		if (clip_x < 0)
		{
			// tiles_idx |=	BG_MAP_FLIP_V;
			// Clip
			size_x += clip_x;
			//size_x = 4;
			// Calculate difference	to switch to next line after copying one
			diff_x = 32	+ size_x;
			// Perform writes
			for	(y = bg_block->size_y; y > 0; y--)
			{
				for	(x = size_x; x > 0;	x--)
					*dst-- = tiles_idx++;
				dst	+= diff_x;
				tiles_idx -= clip_x;
			}
		}
		else
		{
			// Calculate difference	to switch to next line after copying one
			diff_x = 32	+ size_x;
			// Perform writes
			for	(y = bg_block->size_y; y > 0; y--)
			{
				for	(x = size_x; x > 0;	x--)
					*dst-- = tiles_idx++;
				dst	+= diff_x;
			}
		}
	}
	// Non H-Flipped version
	else
	{
		// Setup starting tile index
		tiles_idx =	bg_block->tiles_idx;

		// Perform X clipping
		// (Note: clipping is only done	on left	& bottom)
		clip_x = dst_x + size_x	- 30 ;//32; // FIXME

		if (clip_x > 0)	
		{
			// Clip
			size_x -= clip_x;
			// Calculate difference	to switch to next line after copying one
			diff_x = 32	- size_x;
			// Perform writes
			for	(y = bg_block->size_y; y > 0; y--)
			{
				for	(x = size_x; x > 0;	x--)
					*dst++ = tiles_idx++;
				dst	+= diff_x;
				tiles_idx += clip_x;
			}
		}
		else
		{
			// Calculate difference	to switch to next line after copying one
			diff_x = 32	- size_x;
			// Perform writes
			for	(y = bg_block->size_y; y > 0; y--)
			{
				for	(x = size_x; x > 0;	x--)
					*dst++ = tiles_idx++;
				dst	+= diff_x;
			}
		}
	}
}

// FUNCTION: background_block_clear()
// Clear an area of size given by a block
void		background_block_clear(t_bg *bg, t_bg_block *bg_block, int dst_x, int dst_y, u16 map_value)
{
    u16 *   dst;
    int     size_x, diff_x;
    int     x, y;
    
	// Calculate destination address
	dst = bg->map + (dst_x + (dst_y << 5));

	// Cache original X size
	size_x = bg_block->size_x;

	// Calculate difference	to switch to next line after copying one
	diff_x = 32	- size_x;
	
	// Perform writes
	for	(y = bg_block->size_y; y > 0; y--)
	{
		for	(x = size_x; x > 0;	x--)
			*dst++ = map_value;
		dst	+= diff_x;
	}
}


// FUNCTION: background_block_map_draw()
// Copy given background mapped block to background, at given position
// Note: no clipping is done!
void		background_block_map_draw(t_bg *bg, t_bg_block_map *bg_block_map, int dst_x, int dst_y)
{
    u16 *   dst;
    u16 *   src_map;
    int     tiles_idx_start;
    int     size_x, diff_x;
    int     x, y;

  	// Calculate destination address
	dst = bg->map + (dst_x + (dst_y << 5));
	
	// Cache source map address
	src_map = bg_block_map->map;

	// Cache some values
	size_x = bg_block_map->size_x;
	tiles_idx_start = bg_block_map->tiles_idx;
	
    // Calculate difference	to switch to next line after copying one
	diff_x = 32 - size_x;
    
	// Perform writes
	for (y = bg_block_map->size_y; y > 0; y--)
	{
	    for (x = size_x; x > 0; x--)
	        *dst++ = tiles_idx_start + *src_map++;
	    dst += diff_x;
	}
}

// FUNCTION: background_block_map_clear()
// Clear an area of size given by a mapped block
void		background_block_map_clear(t_bg *bg, t_bg_block_map *bg_block_map, int dst_x, int dst_y, u16 map_value)
{
}

//---------------------------------------------------------------------
