//---------------------------------------------------------------------
// GBALIB: background.h
// by Omar Cornut
//---------------------------------------------------------------------
// Background Handling System
//---------------------------------------------------------------------
// This system is very preliminar.
// It currently only includes basic features needed for Tapper.
// Will probably get an overhaul once a more complex game is done.
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// STRUCTURES
//---------------------------------------------------------------------

// Structure: t_background / t_bg
// Single background instance
typedef struct
{
	u8			number;				/* Background number									*/
	u16			cnt;				/* [Shadow] Register BGxCNT								*/
	u16 *		map;				/* Address of mapping in video memory (5 bits in cnt)	*/
	u16 *		tiles;				/* Address of tiles in video memory (2 bits in cnt)		*/
	u16			scroll_x;			// [Shadow] Scrolling X
	u16			scroll_y;			// [Shadow] Scrolling Y
}				t_background, t_bg;

// Structure: t_background_block / t_bg_block
// Background block of data
typedef struct
{
	u16			tiles_idx;
	u8			size_x;
	u8			size_y;
}				t_background_block, t_bg_block;

// Structure: t_background_block_map / t_bg_block_map
// Background block of mapped data
typedef struct
{
    u16         tiles_idx;
	u16 *		map;
	u8			size_x;
	u8			size_y;
}				t_background_block_map, t_bg_block_map;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------
 
// FUNCTION: background_init()
// Initialize a background
void		background_init(t_bg *bg, int number, u16 cnt);

// FUNCTION: background_sync_cnt()
// Synchronize shadow CNT register with hardware
void		background_sync_cnt(t_bg *bg);

// FUNCTION: background_set_priority()
// Change priority for a given background
void		background_set_priority(t_bg *bg, int priority);

// FUNCTION: background_sync_scroll()
// Synchronize shadowed scrolling values with hardware registers
void		background_sync_scroll(t_bg *bg);

// FUNCTION: background_map_clear()
// Clear background map with given value
void		background_map_clear(t_bg *bg, u16 map_value);

// FUNCTION: background_map_copy()
// Copy given source to background map
void		background_map_copy(t_bg *bg, u16 *map, int flip);

// FUNCTION: background_block_draw()
// Copy given background block to background, at given position
void		background_block_draw(t_bg *bg, t_bg_block *bg_block, int dst_x, int dst_y, int flip);

// FUNCTION: background_block_clear()
// Clear an area of size given by a block
void		background_block_clear(t_bg *bg, t_bg_block *bg_block, int dst_x, int dst_y, u16 map_value);

// FUNCTION: background_block_map_draw()
// Copy given background mapped block to background, at given position
// FIXME: no clipping is done!
void		background_block_map_draw(t_bg *bg, t_bg_block_map *bg_block_map, int dst_x, int dst_y);

// FUNCTION: background_block_map_clear()
// Clear an area of size given by a mapped block
void		background_block_map_clear(t_bg *bg, t_bg_block_map *bg_block_map, int dst_x, int dst_y, u16 map_value);

//---------------------------------------------------------------------
