//---------------------------------------------------------------------
// GBALIB: boundingbox.h
// by Omar Cornut
//---------------------------------------------------------------------
// Bounding Boxes
//---------------------------------------------------------------------
// Note: the current code is based on X1/X2 data only. Usage of Y1/Y2
// will need to implement additionnal (obvious) code here and here.
//---------------------------------------------------------------------

#include "gbalib.h"

//---------------------------------------------------------------------

// FUNCTION: boundingboxes_overlap(t_boundingbox *bb1, t_boundingbox *bb2);
// Tells if two bounding boxes overlap
int			boundingboxes_overlap(t_boundingbox *bb1, t_boundingbox *bb2)
{
	return (bb1->xf1 <= bb2->xf2 && bb1->xf2 >= bb2->xf1);
}

// FUNCTION: boundingboxes_overlap_o(t_boundingbox *bb1, t_boundingbox *bb2, fs32 xf_bb2_offset);
// Tells if two bounding boxes overlap, with a given offset to bb2
int			boundingboxes_overlap_o(t_boundingbox *bb1, t_boundingbox *bb2, fs32 xf_bb2_offset)
{
	return (bb1->xf1 <= bb2->xf2 + xf_bb2_offset && bb1->xf2 >= bb2->xf1 + xf_bb2_offset);
}

// FUNCTION: boundingboxes_overlap_center(t_boundingbox *bb1, t_boundingbox *bb2, fs32 xf_center_offset);
// Tells if bounding box 'bb1' overlap with center+'xcenter_offset' of 'bb2'
int			boundingboxes_overlap_center(t_boundingbox *bb1, t_boundingbox *bb2, fs32 xf_center_offset)
{
	fs32	xf_center;
	
	xf_center = ((bb2->xf1 + bb2->xf2) >> 1) + xf_center_offset;
	return (bb1->xf1 <= xf_center && bb1->xf2 >= xf_center);
}

// FUNCTION: boundingboxes_overlap_center_o(t_boundingbox *bb1, t_boundingbox *bb2, fs32 xf_bb2_offset, fs32 xf_center_offset);
// Tells if bounding box 'bb1' overlap with center+'xcenter_offset' of 'bb2'
int			boundingboxes_overlap_center_o(t_boundingbox *bb1, t_boundingbox *bb2, fs32 xf_bb2_offset, fs32 xf_center_offset)
{
	fs32	xf_center;
	
	xf_center = ((bb2->xf1 + bb2->xf2) >> 1) + xf_bb2_offset + xf_center_offset;
	return (bb1->xf1 <= xf_center && bb1->xf2 >= xf_center);
}

// FUNCTION: boundingbox_cmpx(t_boundingbox *bb, fs32 xf);
// Tells if bounding is before (-1), on (0), or after (+1) given axis line
int			boundingbox_cmpx(t_boundingbox *bb, fs32 xf)
{
	if (bb->xf2 < xf)
		return (-1);
	if (bb->xf1 > xf)
		return (1);
	return (0);
}

// FUNCTION: boundingbox_cmpx_o(t_boundingbox *bb, fs32 xf_bb_offset, fs32 xf);
// Tells if bounding is before (-1), on (0), or after (+1) given axis line
int			boundingbox_cmpx_o(t_boundingbox *bb, fs32 xf_bb_offset, fs32 xf)
{
	if (bb->xf2 + xf_bb_offset < xf)
		return (-1);
	if (bb->xf1 + xf_bb_offset > xf)
		return (1);
	return (0);
}

//---------------------------------------------------------------------
