//---------------------------------------------------------------------
// GBALIB: intrs_vcount.c
// by Omar Cornut
//---------------------------------------------------------------------
// Interrupts System - V-Count
//---------------------------------------------------------------------
// Generic and configurable V-Count handler
//---------------------------------------------------------------------

#include "gbalib.h"

#ifdef INTR_VCOUNT_HANDLER_USE

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

typedef struct
{
	int			line;
	void *		userdata;
}				t_vcount_line;

static t_vcount_line	vcount_lines[INTR_VCOUNT_HANDLER_LINE_BUFFER_SIZE];
static int				vcount_line_current;
static int				vcount_line_setupped;
static void				(*vcount_callback)(void *userdata);

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: intrs_vcount_handler()
// Generic, configurable V-Count handler
void	intrs_vcount_handler(void)
{
	vcount_callback(vcount_lines[vcount_line_current].userdata);
	if (++vcount_line_current == vcount_line_setupped)
		vcount_line_current = 0;
	REG_DISPSTAT = (REG_DISPSTAT & ~REG_DISPSTAT_VCOUNT_Mask) | REG_DISPSTAT_VCOUNT_Set(vcount_lines[vcount_line_current].line);
}

// FUNCTION: intrs_vcount_handler_enable()
// Enable the V-Count handler
void	intrs_vcount_handler_enable(void)
{
	vcount_line_current = 0;
	REG_DISPSTAT = (REG_DISPSTAT & ~REG_DISPSTAT_VCOUNT_Mask) | REG_DISPSTAT_VCOUNT_IE | REG_DISPSTAT_VCOUNT_Set(vcount_lines[0].line);
}

// FUNCTION: intrs_vcount_handler_disable()
// Disable the V-Count handler
void	intrs_vcount_handler_disable(void)
{
	REG_DISPSTAT &= ~REG_DISPSTAT_VCOUNT_IE;
}

// FUNCTION: intrs_vcount_handler_setup_callback()
// Select callback to call on V-Count
void	intrs_vcount_handler_setup_callback(void (*callback)(void *userdata))
{
	vcount_callback = callback;	
}

// FUNCTION: intrs_vcount_handler_setup_clear()
// Clear the list of lines to interrupt at in the V-Count handler
void	intrs_vcount_handler_setup_clear(void)
{
	int	i;
	
	vcount_line_setupped = 0;
	for (i = 0; i < INTR_VCOUNT_HANDLER_LINE_BUFFER_SIZE; i++)
	{
		vcount_lines[i].line = 0;
		vcount_lines[i].userdata = NULL;
	}
}

// FUNCTION: intrs_vcount_handler_setup_addline()
// Add a line and given userdata to the V-Count list of lines
// Those MUST be added sorted as of yet!
void	intrs_vcount_handler_setup_addline(int line, void *userdata)
{
	vcount_lines[vcount_line_setupped].line = line & 0x00FF;
	vcount_lines[vcount_line_setupped].userdata = userdata;
	vcount_line_setupped++;
}

//---------------------------------------------------------------------

#endif /* #ifdef INTR_VCOUNT_HANDLER_USE */
