//---------------------------------------------------------------------
// GBALIB: intrs_vcount.h
// by Omar Cornut
//---------------------------------------------------------------------
// Interrupts System - V-Count
//---------------------------------------------------------------------
// Generic and configurable V-Count handler
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// DEFINITIONS
//---------------------------------------------------------------------

// Define INTR_VCOUNT_HANDLER_USE to enable compilation of the V-Count handler
#define INTR_VCOUNT_HANDLER_USE

// Lines Buffer size for the V-Count handler, each cell will take 4+4 bytes
// If using >= 64 it would be betetr to modify the handler code to use a
// direct lookup table containing the userdata. But then, if you have so
// many V-Count, you'd better handle them yourself directly! (or use H-Blank)
#define INTR_VCOUNT_HANDLER_LINE_BUFFER_SIZE			(8)

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

#ifdef INTR_VCOUNT_HANDLER_USE

// FUNCTION: intrs_vcount_handler()
// Generic, configurable V-Count handler
void	intrs_vcount_handler(void);

// FUNCTION: intrs_vcount_handler_enable/disable()
// Enable/Disable the V-Count handler
void	intrs_vcount_handler_enable(void);
void	intrs_vcount_handler_disable(void);

// FUNCTION: intrs_vcount_handler_setup_callback()
// Select callback to call on V-Count
void	intrs_vcount_handler_setup_callback(void (*callback)(void *userdata));

// FUNCTION: intrs_vcount_handler_setup_clear()
// Clear the list of lines to interrupt at in the V-Count handler
void	intrs_vcount_handler_setup_clear(void);

// FUNCTION: intrs_vcount_handler_setup_addline()
// Add a line and given userdata to the V-Count list of lines
// Those MUST be added sorted as of yet!
void	intrs_vcount_handler_setup_addline(int line, void *userdata);

#endif /* #ifdef INTR_VCOUNT_HANDLER_USE */

//---------------------------------------------------------------------
