//---------------------------------------------------------------------
// GBALIB: windows.h
// by Omar Cornut
//---------------------------------------------------------------------
// Windows Handling System
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// STRUCTURES
//---------------------------------------------------------------------

// Register: t_window_reg
// 6 (8) bits register for display control inside/outside a window
typedef struct
{
    union
    {
        u8          reg;
        struct
        {
            u8      bg0     :1;
            u8      bg1     :1;
            u8      bg2     :1;
            u8      bg3     :1;
            u8      obj     :1;
            u8      effects :1;
            u8      _unused :2;
        };
    };
}                   t_window_reg;

// Structure: t_window_coord
// Window coordinates
typedef struct
{
    union
    {
        u16         x;
        struct
        {
            u8      x2;
            u8      x1;
        };
    };
    union
    {
        u16         y;
        struct
        {
            u8      y2;
            u8      y1;  
        };
    };
}                   t_window_coord;

// Structure: t_window
// Single window
typedef struct
{
    t_window_reg    in;
    t_window_coord  coord;
}                   t_window;

// Structure: t_windows
// Windows Handling System
typedef struct
{
    t_window        win0;
    t_window        win1;
    t_window_reg    wins_out;
    t_window_reg    winobj_in;
}                   t_windows;

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

extern t_windows    windows;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: windows_init()
// Initialize windows handling system
void		windows_init(void);

// FUNCTION: windows_sync()
// Synchronize shadow windows registers data with hardware
void		windows_sync(void);
