//---------------------------------------------------------------------
// SPACE TAPPER: interface_level_select.c
// by Omar Cornut
//---------------------------------------------------------------------
// Interface - Level Select
// Quick and dirty menu
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int		level;
static int		difficulty;

//---------------------------------------------------------------------
// FORWARD DECLARATIONS
//---------------------------------------------------------------------

static void	interface_level_select_refresh(void);
static void	interface_difficulty_select_refresh(void);

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void	    interface_level_select_start(va_list args)
{
	// Setup background 3 (interfaces)
	// -- Priority 0, 256 Colors, 32x32 tiles, Screen base block @ 0x6000D000, Character base block 0
	background_init(&game.bg3, 3, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(0) | REG_BGxCNT_SCREEN_BASE_Set(31));
	background_map_clear(&game.bg3, 0x0000);

	// Setup current level & difficulty
	level = 0;
	difficulty = GAME_DIFFICULTY_NORMAL;
	
	// Display Level Select text interface
	font_bg_printf(&game.bg3, game.font_tiles_start,  0,  0,  PROJECT_NAME "        " PROJECT_DATE);
	font_bg_printf(&game.bg3, game.font_tiles_start, -1,  6, "LEVEL SELECT");
	interface_level_select_refresh();
	font_bg_printf(&game.bg3, game.font_tiles_start, -1, 10, "DIFFICULTY SELECT");
	interface_difficulty_select_refresh();
	font_bg_printf(&game.bg3, game.font_tiles_start,  0, 19, PROJECT_AUTHORS_DESC);

	// Finally, Enable background 3
	REG_DISPCNT |= REG_DISPCNT_BG3_ENABLE;
	
}

void	    interface_level_select_update(void)
{
	if ((game.keys & REG_KEYINPUT_KEY_LEFT) && !(game.keys_old & REG_KEYINPUT_KEY_LEFT))
	{
		level = (level == 0) ? LEVEL_MAX - 1 : level - 1;
		interface_level_select_refresh();
	}
	if ((game.keys & REG_KEYINPUT_KEY_RIGHT) && !(game.keys_old & REG_KEYINPUT_KEY_RIGHT))
	{
		level = (level == LEVEL_MAX - 1) ? 0 : level + 1;
		interface_level_select_refresh();
	}
	if ((game.keys & REG_KEYINPUT_KEY_UP) && !(game.keys_old & REG_KEYINPUT_KEY_UP))
	{
		difficulty = (difficulty == 0) ? GAME_DIFFICULTY_MAX - 1 : difficulty - 1;
		interface_difficulty_select_refresh();
	}
	if ((game.keys & REG_KEYINPUT_KEY_DOWN) && !(game.keys_old & REG_KEYINPUT_KEY_DOWN))
	{
		difficulty = (difficulty == GAME_DIFFICULTY_MAX - 1) ? 0 : difficulty + 1;
		interface_difficulty_select_refresh();
	}
	if (game.keys & REG_KEYINPUT_KEY_START)
	{
		// Setup a new game state
	    game_state_newgame(level, difficulty); /* Note: the first parameter is the starting level there */
	    
	    // Then start at given level
		section_jump(SECTION_GAME_MAIN); 
	}
}

void        interface_level_select_stop(void)
{
	REG_DISPCNT &= ~REG_DISPCNT_BG3_ENABLE;
}

//---------------------------------------------------------------------

static void	interface_level_select_refresh(void)
{
	font_bg_printf(&game.bg3, game.font_tiles_start, -1, 8, "<%02d>", level + 1);
}

static void	interface_difficulty_select_refresh(void)
{
	font_bg_printf(&game.bg3, game.font_tiles_start, -1, 12, (difficulty == GAME_DIFFICULTY_EASY  ) ? "<EASY>"   : " EASY "   );
	font_bg_printf(&game.bg3, game.font_tiles_start, -1, 13, (difficulty == GAME_DIFFICULTY_NORMAL) ? "<NORMAL>" : " NORMAL " );
	font_bg_printf(&game.bg3, game.font_tiles_start, -1, 14, (difficulty == GAME_DIFFICULTY_HARD  ) ? "<HARD>"   : " HARD "   );
}

//---------------------------------------------------------------------
