//---------------------------------------------------------------------
// SPACE TAPPER: sections.c
// by Omar Cornut
//---------------------------------------------------------------------
// Sections
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

t_sections	sections;

static const ts_section sections_data[SECTION_MAX] =
{
	// Section 0: [Main] INTERFACE_TITLE_SCREEN
	{
		interface_title_start,
		interface_title_update,
		interface_title_stop
	},
	// Section 1: [Main] INTERFACE_LEVEL_SELECT
	{
		interface_level_select_start,
		interface_level_select_update,
		interface_level_select_stop
	},
	// Section 2: [Main] INTERFACE_GAME_OVER
	{
		interface_game_over_start,
		interface_game_over_update,
		interface_game_over_stop
	},
	// Section 3: [Main] INTERFACE_LEVEL_END
	{
		interface_level_end_start,
		interface_level_end_update,
		interface_level_end_stop
	},
	// Section 4: [Main] GAME_MAIN
	{
		game_main_start,
		game_main_update,
		game_main_stop
	},
	// Section 5: [Sub]  EFFECTS
	{
		effects_start,
		effects_update,
		effects_stop
	}
};

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void    sections_init(void)
{
    sections.stack_pos = 0;
    sections.current = NULL;
}

//---------------------------------------------------------------------

void	section_jump(int id, ...)
{
	if (sections.current)
	{
		if (sections.current->data->f_stop)
			sections.current->data->f_stop();
	}
	else
	{
		sections.current = &sections.stack[sections.stack_pos];
	}
	sections.current->id = id;
	sections.current->data = (ts_section *)&sections_data[id];
	if (sections.current->data->f_start)
	{
		va_list args;
		va_start(args, id);
		sections.current->data->f_start(args);
		va_end(args);
	}
}

void	section_call(int id, ...)
{
	sections.stack_pos++;
	if (sections.stack_pos >= SECTIONS_STACK_SIZE)
	{ 
		/* FIXME: fatal error */ 
		return;
	}
	sections.current++;
	sections.current->id = id;
	sections.current->data = (ts_section *)&sections_data[id];	
	if (sections.current->data->f_start)
	{
		va_list args;
		va_start(args, id);
		sections.current->data->f_start(args);
		va_end(args);
	}
}

void	section_return(void)
{
	if (sections.current && sections.current->data->f_stop)
		sections.current->data->f_stop();
	sections.stack_pos--;
	if (sections.stack < 0)
	{
		/* FIXME: fatal */
		return;
	}
	sections.current--;	
}

//---------------------------------------------------------------------
