//---------------------------------------------------------------------
// ANIM-To-C: animtoc.c
// by Omar Cornut, 2002
//---------------------------------------------------------------------
// Main Program File
//---------------------------------------------------------------------

#include "animtoc.h"

//---------------------------------------------------------------------

int			animtoc_check(t_anim_set *a)
{
	int		error = 0;
	
	if (a->symbol_name == NULL)
		error = 1, animtoc_error("SymbolName not defined!");
	if (a->picture_size_pixels_x == 0 || a->picture_size_pixels_y == 0)
		error = 1, animtoc_error("PictureSizePixels not (properly) defined!");
	if (a->picture_size_bytes == 0)
		error = 1, animtoc_error("PictureSizeBytes not defined!");
	if (a->picture_bpp == 0)
		error = 1, animtoc_error("PictureBPP not defined!");
	if (a->positions_data == NULL)
		error = 1, animtoc_error("No Position defined. Need at least one!");
	if (a->boundingboxes_data == NULL)
		error = 1, animtoc_error("No BoundingBox defined. Need at least one!");
	if (a->anims_data == NULL)
		error = 1, animtoc_error("No Animation defined. Need at least one, or this file is worthless...");
	if ((a->picture_size_pixels_x * a->picture_size_pixels_y * a->picture_bpp) / 8 != a->picture_size_bytes)
		error = 1, animtoc_error("PictureSizeBytes does not correspond to PictureSizePixels*PictureBPP!");
	if (error != 0)
	{
		animtoc_error("Quitting due to at least one error.");
		return (error);
	}
	return (0);
}

//---------------------------------------------------------------------

int		encode_shift_v = 0;
#define encode_shift() printf("%-*s", encode_shift_v, "")

void	encode(int value, int size)
{
	anim_set.output_pos += size; /* FIXME: access to anim_set */
	while (size > 0)
	{
		printf("0x%02x, ", value & 0xFF);
		value >>= 8;
		size--;
	}
}

//---------------------------------------------------------------------

int			animtoc_print(t_anim_set *a)
{
	if (animtoc_check(a))
		return (1);

	printf("/*\n");
	printf("** Animation Data\n");
	printf("** Generated from \"%s\" by ANIM-To-C (c) Omar Cornut 2002\n", a->source_file_name);
	printf("*/\n");
	printf("\n");
	printf("const u8\t\t%s_set[] __attribute__((aligned(4))) =\n", a->symbol_name);
	printf("{\n");

	// FIXME: this sucks
	a->anims_offsets_offset = 5 * sizeof(u16); // sizeof(ts_anim_header)
	a->anims_num = list_size(a->anims_data);
	
	a->positions_offset = a->anims_offsets_offset + a->anims_num * sizeof(u16);
	a->positions_num = list_size(a->positions_data);
	
	a->boundingboxes_offset = a->positions_offset + a->positions_num * 2;
	a->boundingboxes_num = list_size(a->boundingboxes_data);

	a->anims_offset = a->boundingboxes_offset + a->boundingboxes_num * ANIM_BOUNDINGBOX_SIZE * ANIM_BOUNDINGBOX_FLIP_MAX;

	a->output_pos = 0;

	encode_shift_v = 2;

	encode_shift(); printf("//---- Byte %04d ----\n", a->output_pos);
	encode_shift(); printf("// Picture Size (%d bytes)\n", a->picture_size_bytes);
	encode_shift(); encode(a->picture_size_bytes, 2); printf("\n");
	encode_shift();	printf("// Positions Data Size\n");
	encode_shift(); encode(a->positions_num, 2); printf("\n");
	encode_shift();	printf("// Positions Data Offset\n");
	encode_shift(); encode(a->positions_offset, 2); printf("\n");
	encode_shift(); printf("// Bounding Boxes Data Size\n");
	encode_shift(); encode(a->boundingboxes_num, 2); printf("\n");
	encode_shift(); printf("// Bounding Boxes Data Offset\n");
	encode_shift(); encode(a->boundingboxes_offset, 2); printf("\n");
	printf("\n");

	// Animations Offsets
	{
		int			i;
		t_list *	anims;
	
		encode_shift(); printf("//---- Byte %04d ----\n", a->output_pos);
		encode_shift(); printf("// Animations Offsets\n");
		encode_shift_v += 2;
		for (i = 0, anims = a->anims_data; anims != NULL; i++, anims = anims->next)
		{
			t_anim *anim = anims->elem;
			encode_shift();	printf("// Animation %d Offset\n", i);
			encode_shift();	encode(anim->pointer + a->anims_offset, 2); printf("\n");
		}
		encode_shift_v -= 2;
	}
	printf("\n");

	// Positions
    {
        t_list *        positions;

		encode_shift(); printf("//---- Byte %04d ----\n", a->output_pos);
		encode_shift(); printf("// Positions Data\n");
        encode_shift_v += 2;
        for (positions = a->positions_data; positions != NULL; positions = positions->next)
        {
            t_position *position = positions->elem;
            encode_shift();	printf("// Position Data %d\n", position->index);
            encode_shift();	printf("%d, %d,\n", position->x, position->y);
            a->output_pos += 2;
        }
        encode_shift_v -= 2;
    }
	printf("\n");

	// Bounding Boxes
    {
        t_list *        boundingboxes;

		encode_shift(); printf("//---- Byte %04d ----\n", a->output_pos);
		encode_shift(); printf("// Bounding Boxes Data\n");
		encode_shift_v += 2;
        for (boundingboxes = a->boundingboxes_data; boundingboxes != NULL; boundingboxes = boundingboxes->next)
        {
        	int flip;
            t_boundingbox *boundingbox = boundingboxes->elem;
            int size_x = boundingbox->size_x ? boundingbox->size_x : a->picture_size_pixels_x;
            encode_shift();	printf("// Bounding Box Data %d\n", boundingbox->index);
            //encode_shift();	printf("%d, %d, %d, %d,\n", boundingbox->x1, boundingbox->y1, boundingbox->x2, boundingbox->y2);
            for (flip = 0; flip < ANIM_BOUNDINGBOX_FLIP_MAX; flip++)
            {
            	signed short v1, v2;
            	char *		 desc;
            	switch (flip)
            	{
            		case 0:	v1 = boundingbox->x1;
            				v2 = boundingbox->x2;
            				if (ANIM_BOUNDINGBOX_FLIP_MAX > 1)
            					desc = "No flip";
            				else
            					desc = "";
            				break;
            		case 1:	v1 = size_x - 1 - boundingbox->x2; // X1 & X2 inverted
            				v2 = size_x - 1 - boundingbox->x1;
            				desc = "X flip";
            				break;
            		default:
            				desc = NULL; // Will crash
            				break;
            	}
            	// Convert to fixed 12.4
            	v1 <<= 4;
            	v2 <<= 4;
            	// Print
            	encode_shift();	encode(v1, 2); encode(v2, 2); printf("\t// (%d.0,%d.0)\t(%s)\n", v1 >> 4, v2 >> 4, desc);
            }
        }
        encode_shift_v -= 2;
    }
	printf("\n");

	// Animations Data
	{
		int			i;
		t_list *	anims;

		encode_shift(); printf("//---- Byte %04d ----\n", a->output_pos);
		encode_shift(); printf("// Animations Data\n");
		encode_shift_v += 2;
		for (i = 0, anims = a->anims_data; anims != NULL; i++, anims = anims->next)
		{
			int 			j;
			t_anim *		anim;
			t_list *		frames;
    	
			anim = anims->elem;
			encode_shift();	printf("// Animation %d\n", i);
			{
				encode_shift_v += 2;
				encode_shift();	printf("// Next (Anim/Frame)\n");
				encode_shift();	printf("%d, %d,\n", anim->next_anim, anim->next_frame);
				a->output_pos += 2;
				for (j = 0, frames = anim->frames; frames != NULL; j++, frames = frames->next)
				{
					t_anim_frame *frame = frames->elem;
					encode_shift();	printf("// Frame %d -> Delay %d, Picture %d", j, frame->delay, frame->i_picture);
					if (frame->i_offset)
					    printf(", Offset %d", frame->i_offset);
					if (frame->i_boundingbox)
						printf(", BoundingBox %d", frame->i_boundingbox);
					printf("\n");
					encode_shift();	printf("%d, %d, %d, %d,\n", frame->delay, frame->i_picture, frame->i_offset, frame->i_boundingbox);
					a->output_pos += 4;
				}
				encode_shift();	printf("// Frames End\n");
				encode_shift();	printf("0,\n");
				a->output_pos += 1;
				encode_shift_v -= 2;
			}
			if (anims->next)
				printf("\n");
		}
		encode_shift_v -= 2;
	}

	encode_shift(); printf("//---- Byte %04d ----\n", a->output_pos);
	encode_shift_v = 0;
	printf("};\n");

	return (0);
}

//---------------------------------------------------------------------

void		animtoc_init(t_anim_set *a, char *source_file_name)
{
	a->source_file_name = strdup(source_file_name);
  	a->symbol_name = NULL;
  	a->picture_size_pixels_x = 0;
  	a->picture_size_pixels_y = 0;
  	a->picture_size_bytes = 0;
  	a->picture_bpp = 0;
  	// Positions
  	a->positions_data = NULL;
  	a->positions_offset = 0;
  	a->positions_num = 0;
  	// Bounding Boxes
  	a->boundingboxes_data = NULL;
  	a->boundingboxes_offset = 0;
  	a->boundingboxes_num = 0;
  	// Animations
  	a->anims_data = NULL;
  	a->anims_offsets_offset = 0;
  	a->anims_offset = 0;
  	a->anims_num = 0;
	// Work
  	a->output_pos = 0;
  	anim_current = NULL;
  	anim_frame_current = NULL;
}

//---------------------------------------------------------------------

void		animtoc_error(char *format, ...)
{
	va_list	args;
	
	va_start(args, format);
	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
	va_end(args);
}

void		animtoc_fatal(char *format, ...)
{
	va_list	args;
	
	va_start(args, format);
	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
	va_end(args);
	exit(1);
}

//---------------------------------------------------------------------

int			main(int argc, char **argv)
{
	if (argc < 2)
	{
		printf("Syntax: %s <file.a>\n", argv[0]);
		return (0);
	}

	// yydebug = 1
	yyin = fopen(argv[1], "rt");
	if (yyin == NULL)
	{
		perror(argv[1]);
		return (1);
	}

	animtoc_init(&anim_set, argv[1]);
	yyparse();
	if (animtoc_print(&anim_set))
		return (1);
	return (0);
}

//---------------------------------------------------------------------
